@@ -1,8 +1,19 @@
 open Stdune
 
 include module type of struct
-  include Cmdliner.Arg
-end
+    include Cmdliner.Arg
+  end
+  with type info := Cmdliner.Arg.info
+
+val info
+  :  ?deprecated:string
+  -> ?absent:string
+  -> ?docs:string
+  -> ?docv:string
+  -> doc:string option
+  -> ?env:Cmdliner.Cmd.Env.info
+  -> string list
+  -> Cmdliner.Arg.info
 
 module Path : sig
   module External : sig
