@@ -17,10 +17,43 @@ open struct
   module Only_packages = Only_packages
 end
 
+module Workspace = Source.Workspace
+
 open struct
   open Cmdliner
   module Cmd = Cmd
-  module Term = Term
+
+  module Term = struct
+    include Term
+
+    (* Evaluate a parser, passing it no command-line arguments, in an
+       environment with no variables set. It's only valid to pass a parser
+       which can be evaluated with no arguments, otherwise a code error will
+       be raised. Returns the result of the parser. This is intended to be
+       used to extract the default value for a type implied by the behaviour
+       of its parser when no command-line arguments are passed to it. *)
+    let eval_no_args_empty_env t =
+      let raise_code_error data =
+        Code_error.raise "Unexpected result evaluating term with no args" data
+      in
+      (* Cmdliner doesn't allow argv to be empty. *)
+      let argv = [| "dune" |] in
+      let env _ = None in
+      match Cmd.eval_value ~argv ~env (Cmd.v (Cmd.info "dune") t) with
+      | Ok (`Ok x) -> x
+      | Ok `Help -> raise_code_error [ "ok", Dyn.string "help" ]
+      | Ok `Version -> raise_code_error [ "ok", Dyn.string "version" ]
+      | Error e ->
+        let error_string =
+          match e with
+          | `Parse -> "parse"
+          | `Term -> "term"
+          | `Exn -> "exn"
+        in
+        raise_code_error [ "error", Dyn.string error_string ]
+    ;;
+  end
+
   module Manpage = Manpage
 end
 
@@ -42,7 +75,7 @@ let debug_backtraces =
     & info
         [ "debug-backtraces" ]
         ~docs:copts_sect
-        ~doc:"Always print exception backtraces.")
+        ~doc:(Some "Always print exception backtraces."))
 ;;
 
 let default_build_dir = "_build"
@@ -61,7 +94,9 @@ let one_of term1 term2 =
 let build_info =
   let+ build_info =
     Arg.(
-      value & flag & info [ "build-info" ] ~docs:"OPTIONS" ~doc:"Show build information.")
+      value
+      & flag
+      & info [ "build-info" ] ~docs:"OPTIONS" ~doc:(Some "Show build information."))
   in
   if build_info
   then (
@@ -121,14 +156,14 @@ module Options_implied_by_dash_p = struct
                  [ "config-file" ]
                  ~docs
                  ~docv:"FILE"
-                 ~doc:"Load this configuration file instead of the default one.")
+                 ~doc:(Some "Load this configuration file instead of the default one."))
          in
          Option.map fn ~f:(fun fn -> This (Arg.Path.path fn)))
         (let+ x =
            Arg.(
              value
              & flag
-             & info [ "no-config" ] ~docs ~doc:"Do not load the configuration file")
+             & info [ "no-config" ] ~docs ~doc:(Some "Do not load the configuration file"))
          in
          Option.some_if x No_config)
     in
@@ -165,6 +200,11 @@ module Options_implied_by_dash_p = struct
 
   let options =
     let+ root =
+      let doc =
+        "Use this directory as workspace root instead of guessing it. Note that this \
+         option doesn't change the interpretation of targets given on the command line. \
+         It is only intended for scripts."
+      in
       Arg.(
         value
         & opt (some dir) None
@@ -172,10 +212,8 @@ module Options_implied_by_dash_p = struct
             [ "root" ]
             ~docs
             ~docv:"DIR"
-            ~doc:
-              "Use this directory as workspace root instead of guessing it. Note that \
-               this option doesn't change the interpretation of targets given on the \
-               command line. It is only intended for scripts.")
+            ~doc:(Some doc)
+            ~env:(Cmd.Env.info ~doc "DUNE_ROOT"))
     and+ ignore_promoted_rules =
       Arg.(
         value
@@ -184,9 +222,10 @@ module Options_implied_by_dash_p = struct
             [ "ignore-promoted-rules" ]
             ~docs
             ~doc:
-              "Ignore rules with (mode promote), except ones with (only ...). The \
-               variable %{ignoring_promoted_rules} in dune files reflects whether this \
-               option was passed or not.")
+              (Some
+                 "Ignore rules with (mode promote), except ones with (only ...). The \
+                  variable %{ignoring_promoted_rules} in dune files reflects whether \
+                  this option was passed or not."))
     and+ config_from_config_file = config_term
     and+ default_target =
       Arg.(
@@ -197,26 +236,27 @@ module Options_implied_by_dash_p = struct
             ~docs
             ~docv:"TARGET"
             ~doc:
-              "Set the default target that is used when none is specified to $(b,dune \
-               build).")
+              (Some
+                 "Set the default target that is used when none is specified to $(b,dune \
+                  build)."))
     and+ always_show_command_line =
       let doc = "Always show the full command lines of programs executed by dune." in
-      Arg.(value & flag & info [ "always-show-command-line" ] ~docs ~doc)
+      Arg.(value & flag & info [ "always-show-command-line" ] ~docs ~doc:(Some doc))
     and+ promote_install_files =
       let doc = "Promote any generated <package>.install files to the source tree." in
       Arg.(
         last
         & opt_all ~vopt:true bool [ false ]
-        & info [ "promote-install-files" ] ~docs ~doc)
+        & info [ "promote-install-files" ] ~docs ~doc:(Some doc))
     and+ require_dune_project_file =
       let doc = "Fail if a dune-project file is missing." in
       Arg.(
         last
         & opt_all ~vopt:true bool [ false ]
-        & info [ "require-dune-project-file" ] ~docs ~doc)
+        & info [ "require-dune-project-file" ] ~docs ~doc:(Some doc))
     and+ ignore_lock_dir =
       let doc = "Ignore dune.lock/ directory." in
-      Arg.(value & flag & info [ "ignore-lock-dir" ] ~docs ~doc)
+      Arg.(value & flag & info [ "ignore-lock-dir" ] ~docs ~doc:(Some doc))
     in
     { root
     ; only_packages = No_restriction
@@ -254,12 +294,13 @@ module Options_implied_by_dash_p = struct
           ~docs
           ~docv:"PACKAGES"
           ~doc:
-            (sprintf
-               "Put $(b,dune) into a reproducible $(i,release) mode. Shorthand for \
-                $(b,%s). You should use this option for release builds. For instance, \
-                you must use this option in your $(i,<package>.opam) files. Except if \
-                you already use $(b,-p), as $(b,-p) implies this option."
-               (String.concat ~sep:" " shorthand_for)))
+            (Some
+               (sprintf
+                  "Put $(b,dune) into a reproducible $(i,release) mode. Shorthand for \
+                   $(b,%s). You should use this option for release builds. For instance, \
+                   you must use this option in your $(i,<package>.opam) files. Except if \
+                   you already use $(b,-p), as $(b,-p) implies this option."
+                  (String.concat ~sep:" " shorthand_for))))
   ;;
 
   let options =
@@ -275,12 +316,13 @@ module Options_implied_by_dash_p = struct
               ~docs
               ~docv:"PACKAGES"
               ~doc:
-                "Ignore stanzas referring to a package that is not in $(b,PACKAGES). \
-                 $(b,PACKAGES) is a comma-separated list of package names. Note that \
-                 this has the same effect as deleting the relevant stanzas from dune \
-                 files. It is mostly meant for releases. During development, it is \
-                 likely that what you want instead is to build a particular \
-                 $(b,<package>.install) target.")
+                (Some
+                   "Ignore stanzas referring to a package that is not in $(b,PACKAGES). \
+                    $(b,PACKAGES) is a comma-separated list of package names. Note that \
+                    this has the same effect as deleting the relevant stanzas from dune \
+                    files. It is mostly meant for releases. During development, it is \
+                    likely that what you want instead is to build a particular \
+                    $(b,<package>.install) target."))
       in
       match names with
       | None -> Only_packages.Clflags.No_restriction
@@ -299,10 +341,11 @@ module Options_implied_by_dash_p = struct
             ~docs
             ~docv:"PACKAGES"
             ~doc:
-              "Shorthand for $(b,--release --only-packages PACKAGE). You must use this \
-               option in your $(i,<package>.opam) files, in order to build only what's \
-               necessary when your project contains multiple packages as well as getting \
-               reproducible builds.")
+              (Some
+                 "Shorthand for $(b,--release --only-packages PACKAGE). You must use \
+                  this option in your $(i,<package>.opam) files, in order to build only \
+                  what's necessary when your project contains multiple packages as well \
+                  as getting reproducible builds."))
   ;;
 
   let term =
@@ -318,10 +361,11 @@ module Options_implied_by_dash_p = struct
             ~docs
             ~env:(Cmd.Env.info ~doc "DUNE_PROFILE")
             ~doc:
-              (Printf.sprintf
-                 "Select the build profile, for instance $(b,dev) or $(b,release). The \
-                  default is $(b,%s)."
-                 (Profile.to_string Profile.default)))
+              (Some
+                 (Printf.sprintf
+                    "Select the build profile, for instance $(b,dev) or $(b,release). \
+                     The default is $(b,%s)."
+                    (Profile.to_string Profile.default))))
     in
     match profile with
     | None -> t
@@ -336,7 +380,10 @@ let display_term =
        Arg.(
          value
          & flag
-         & info [ "verbose" ] ~docs:copts_sect ~doc:"Same as $(b,--display verbose)")
+         & info
+             [ "verbose" ]
+             ~docs:copts_sect
+             ~doc:(Some "Same as $(b,--display verbose)"))
      in
      Option.some_if verbose Dune_config.Display.verbose)
     Arg.(
@@ -349,7 +396,7 @@ let display_term =
       in
       value
       & opt (some (enum Display.all)) None
-      & info [ "display" ] ~docs:copts_sect ~docv:"MODE" ~doc)
+      & info [ "display" ] ~docs:copts_sect ~docv:"MODE" ~doc:(Some doc))
 ;;
 
 let shared_with_config_file =
@@ -368,7 +415,7 @@ let shared_with_config_file =
           [ "j" ]
           ~docs
           ~docv:"JOBS"
-          ~doc:"Run no more than $(i,JOBS) commands simultaneously.")
+          ~doc:(Some "Run no more than $(i,JOBS) commands simultaneously."))
   and+ sandboxing_preference =
     let all =
       List.map Dune_engine.Sandbox_mode.all_except_patch_back_source_tree ~f:(fun s ->
@@ -384,14 +431,15 @@ let shared_with_config_file =
                ~doc:"Sandboxing mode to use by default. (see --sandbox)"
                "DUNE_SANDBOX")
           ~doc:
-            (Printf.sprintf
-               "Set sandboxing mode. Some actions require a certain sandboxing mode, so \
-                they will ignore this setting. The allowed values are: %s."
-               (String.concat
-                  ~sep:", "
-                  (List.map
-                     Dune_engine.Sandbox_mode.all_except_patch_back_source_tree
-                     ~f:Dune_engine.Sandbox_mode.to_string))))
+            (Some
+               (Printf.sprintf
+                  "Set sandboxing mode. Some actions require a certain sandboxing mode, \
+                   so they will ignore this setting. The allowed values are: %s."
+                  (String.concat
+                     ~sep:", "
+                     (List.map
+                        Dune_engine.Sandbox_mode.all_except_patch_back_source_tree
+                        ~f:Dune_engine.Sandbox_mode.to_string)))))
   and+ terminal_persistence =
     let modes = Dune_config.Terminal_persistence.all in
     let doc =
@@ -404,7 +452,7 @@ let shared_with_config_file =
     Arg.(
       value
       & opt (some (enum modes)) None
-      & info [ "terminal-persistence" ] ~docs ~docv:"MODE" ~doc)
+      & info [ "terminal-persistence" ] ~docs ~docv:"MODE" ~doc:(Some doc))
   and+ display = display_term
   and+ cache_enabled =
     let doc =
@@ -416,7 +464,7 @@ let shared_with_config_file =
     Arg.(
       value
       & opt (some (enum Dune_config.Cache.Toggle.all)) None
-      & info [ "cache" ] ~docs ~env:(Cmd.Env.info ~doc "DUNE_CACHE") ~doc)
+      & info [ "cache" ] ~docs ~env:(Cmd.Env.info ~doc "DUNE_CACHE") ~doc:(Some doc))
   and+ cache_storage_mode =
     let doc =
       Printf.sprintf
@@ -431,7 +479,7 @@ let shared_with_config_file =
           [ "cache-storage-mode" ]
           ~docs
           ~env:(Cmd.Env.info ~doc "DUNE_CACHE_STORAGE_MODE")
-          ~doc)
+          ~doc:(Some doc))
   and+ cache_check_probability =
     let doc =
       Printf.sprintf
@@ -447,7 +495,7 @@ let shared_with_config_file =
           [ "cache-check-probability" ]
           ~docs
           ~env:(Cmd.Env.info ~doc "DUNE_CACHE_CHECK_PROBABILITY")
-          ~doc)
+          ~doc:(Some doc))
   and+ action_stdout_on_success =
     Arg.(
       value
@@ -455,12 +503,13 @@ let shared_with_config_file =
       & info
           [ "action-stdout-on-success" ]
           ~doc:
-            "Specify how to deal with the standard output of actions when they succeed. \
-             Possible values are: $(b,print) to just print it to Dune's output, \
-             $(b,swallow) to completely ignore it and $(b,must-be-empty) to enforce that \
-             the action printed nothing. With $(b,must-be-empty), Dune will consider \
-             that the action failed if it printed something to its standard output. The \
-             default is $(b,print).")
+            (Some
+               "Specify how to deal with the standard output of actions when they \
+                succeed. Possible values are: $(b,print) to just print it to Dune's \
+                output, $(b,swallow) to completely ignore it and $(b,must-be-empty) to \
+                enforce that the action printed nothing. With $(b,must-be-empty), Dune \
+                will consider that the action failed if it printed something to its \
+                standard output. The default is $(b,print)."))
   and+ action_stderr_on_success =
     Arg.(
       value
@@ -468,11 +517,12 @@ let shared_with_config_file =
       & info
           [ "action-stderr-on-success" ]
           ~doc:
-            "Same as $(b,--action-stdout-on-success) but for standard error instead of \
-             standard output. A good default for large mono-repositories is \
-             $(b,--action-stdout-on-success=swallow \
-             --action-stderr-on-success=must-be-empty). This ensures that a successful \
-             build has a \"clean\" empty output.")
+            (Some
+               "Same as $(b,--action-stdout-on-success) but for standard error instead \
+                of standard output. A good default for large mono-repositories is \
+                $(b,--action-stdout-on-success=swallow \
+                --action-stderr-on-success=must-be-empty). This ensures that a \
+                successful build has a \"clean\" empty output."))
   in
   { Dune_config.Partial.display
   ; concurrency
@@ -487,6 +537,7 @@ let shared_with_config_file =
   ; action_stdout_on_success
   ; action_stderr_on_success
   ; project_defaults = None
+  ; pkg_enabled = None
   ; experimental = None
   }
 ;;
@@ -543,11 +594,12 @@ let cache_debug_flags_term : Cache_debug_flags.t Term.t =
           [ "debug-cache" ]
           ~docs:copts_sect
           ~doc:
-            (sprintf
-               "Show debug messages on cache misses for the given cache layers. Value is \
-                a comma-separated list of cache layer names. All available cache layers: \
-                %s."
-               all_layer_names))
+            (Some
+               (sprintf
+                  "Show debug messages on cache misses for the given cache layers. Value \
+                   is a comma-separated list of cache layer names. All available cache \
+                   layers: %s."
+                  all_layer_names)))
   in
   value initial
 ;;
@@ -580,7 +632,7 @@ module Builder = struct
     ; always_show_command_line : bool
     ; promote_install_files : bool
     ; file_watcher : Dune_engine.Scheduler.Run.file_watcher
-    ; workspace_config : Dune_rules.Workspace.Clflags.t
+    ; workspace_config : Workspace.Clflags.t
     ; cache_debug_flags : Dune_engine.Cache_debug_flags.t
     ; report_errors_config : Dune_engine.Report_errors_config.t
     ; separate_error_messages : bool
@@ -590,10 +642,9 @@ module Builder = struct
     ; build_dir : string
     ; root : string option
     ; stats_trace_file : string option
-    ; stats_trace_extended : bool
     ; allow_builds : bool
     ; default_root_is_cwd : bool
-    ; log_file : Dune_util.Log.File.t
+    ; log_file : Log.File.t
     }
 
   let root t = t.root
@@ -632,8 +683,9 @@ module Builder = struct
             [ "debug-dependency-path" ]
             ~docs
             ~doc:
-              "In case of error, print the dependency path from the targets on the \
-               command line to the rule that failed.")
+              (Some
+                 "In case of error, print the dependency path from the targets on the \
+                  command line to the rule that failed."))
     and+ debug_backtraces = debug_backtraces
     and+ debug_artifact_substitution =
       Arg.(
@@ -642,7 +694,7 @@ module Builder = struct
         & info
             [ "debug-artifact-substitution" ]
             ~docs
-            ~doc:"Print debugging info about artifact substitution")
+            ~doc:(Some "Print debugging info about artifact substitution"))
     and+ debug_load_dir =
       Arg.(
         value
@@ -650,7 +702,7 @@ module Builder = struct
         & info
             [ "debug-load-dir" ]
             ~docs
-            ~doc:"Print debugging info about directory loading")
+            ~doc:(Some "Print debugging info about directory loading"))
     and+ debug_digests =
       Arg.(
         value
@@ -658,7 +710,7 @@ module Builder = struct
         & info
             [ "debug-digests" ]
             ~docs
-            ~doc:"Explain why Dune decides to re-digest some files")
+            ~doc:(Some "Explain why Dune decides to re-digest some files"))
     and+ debug_package_logs =
       let doc = "Always print the standard logs when building packages" in
       Arg.(
@@ -667,7 +719,7 @@ module Builder = struct
         & info
             [ "debug-package-logs" ]
             ~docs
-            ~doc
+            ~doc:(Some doc)
             ~env:(Cmd.Env.info ~doc "DUNE_DEBUG_PACKAGE_LOGS"))
     and+ no_buffer =
       let doc =
@@ -680,7 +732,7 @@ module Builder = struct
          interleaving. Additionally you should use $(b,--verbose) as well, to make sure \
          that commands are printed before they are being executed."
       in
-      Arg.(value & flag & info [ "no-buffer" ] ~docs ~docv:"DIR" ~doc)
+      Arg.(value & flag & info [ "no-buffer" ] ~docs ~docv:"DIR" ~doc:(Some doc))
     and+ workspace_file =
       let doc = "Use this specific workspace file instead of looking it up." in
       Arg.(
@@ -690,7 +742,7 @@ module Builder = struct
             [ "workspace" ]
             ~docs
             ~docv:"FILE"
-            ~doc
+            ~doc:(Some doc)
             ~env:(Cmd.Env.info ~doc "DUNE_WORKSPACE"))
     and+ promote =
       one_of
@@ -702,14 +754,15 @@ module Builder = struct
                  [ "auto-promote" ]
                  ~docs
                  ~doc:
-                   "Automatically promote files. This is similar to running $(b,dune \
-                    promote) after the build.")
+                   (Some
+                      "Automatically promote files. This is similar to running $(b,dune \
+                       promote) after the build."))
          in
          Option.some_if auto Dune_engine.Clflags.Promote.Automatically)
         (let+ disable =
            let doc = "Disable all promotion rules" in
            let env = Cmd.Env.info ~doc "DUNE_DISABLE_PROMOTION" in
-           Arg.(value & flag & info [ "disable-promotion" ] ~docs ~env ~doc)
+           Arg.(value & flag & info [ "disable-promotion" ] ~docs ~env ~doc:(Some doc))
          in
          Option.some_if disable Dune_engine.Clflags.Promote.Never)
     and+ force =
@@ -719,8 +772,9 @@ module Builder = struct
         & info
             [ "force"; "f" ]
             ~doc:
-              "Force actions associated to aliases to be re-executed even if their \
-               dependencies haven't changed.")
+              (Some
+                 "Force actions associated to aliases to be re-executed even if their \
+                  dependencies haven't changed."))
     and+ watch =
       let+ res =
         one_of
@@ -731,8 +785,9 @@ module Builder = struct
                & info
                    [ "watch"; "w" ]
                    ~doc:
-                     "Instead of terminating build after completion, wait continuously \
-                      for file changes.")
+                     (Some
+                        "Instead of terminating build after completion, wait \
+                         continuously for file changes."))
            in
            if watch then Some Dune_rpc_impl.Watch_mode_config.Eager else None)
           (let+ watch =
@@ -742,8 +797,9 @@ module Builder = struct
                & info
                    [ "passive-watch-mode" ]
                    ~doc:
-                     "Similar to [--watch], but only start a build when instructed \
-                      externally by an RPC.")
+                     (Some
+                        "Similar to [--watch], but only start a build when instructed \
+                         externally by an RPC."))
            in
            if watch then Some Dune_rpc_impl.Watch_mode_config.Passive else None)
       in
@@ -757,7 +813,7 @@ module Builder = struct
         & info
             [ "print-metrics" ]
             ~docs
-            ~doc:"Print out various performance metrics after every build.")
+            ~doc:(Some "Print out various performance metrics after every build."))
     and+ dump_memo_graph_file =
       Arg.(
         value
@@ -766,7 +822,7 @@ module Builder = struct
             [ "dump-memo-graph" ]
             ~docs
             ~docv:"FILE"
-            ~doc:"Dump the dependency graph to a file after the build is complete.")
+            ~doc:(Some "Dump the dependency graph to a file after the build is complete."))
     and+ dump_memo_graph_format =
       Arg.(
         value
@@ -775,7 +831,7 @@ module Builder = struct
             [ "dump-memo-graph-format" ]
             ~docs
             ~docv:"FORMAT"
-            ~doc:"Set the file format used by $(b,--dump-memo-graph)")
+            ~doc:(Some "Set the file format used by $(b,--dump-memo-graph)"))
     and+ dump_memo_graph_with_timing =
       Arg.(
         value
@@ -784,10 +840,11 @@ module Builder = struct
             [ "dump-memo-graph-with-timing" ]
             ~docs
             ~doc:
-              "Re-run each cached node in the Memo graph after building and include the \
-               run duration in the output of $(b,--dump-memo-graph). Since all nodes \
-               contain a cached value, each measurement will only account for a single \
-               node.")
+              (Some
+                 "Re-run each cached node in the Memo graph after building and include \
+                  the run duration in the output of $(b,--dump-memo-graph). Since all \
+                  nodes contain a cached value, each measurement will only account for a \
+                  single node."))
     and+ dump_gc_stats =
       Arg.(
         value
@@ -796,7 +853,9 @@ module Builder = struct
             [ "dump-gc-stats" ]
             ~docs
             ~docv:"FILE"
-            ~doc:"Dump the garbage collector stats to a file after the build is complete.")
+            ~doc:
+              (Some
+                 "Dump the garbage collector stats to a file after the build is complete."))
     and+ { Options_implied_by_dash_p.root
          ; only_packages
          ; ignore_promoted_rules
@@ -814,7 +873,7 @@ module Builder = struct
       Arg.(
         value
         & opt (some Arg.context_name) None
-        & info [ "x" ] ~docs ~doc:"Cross-compile using this toolchain.")
+        & info [ "x" ] ~docs ~doc:(Some "Cross-compile using this toolchain."))
     and+ build_dir =
       let doc = "Specified build directory. _build if unspecified" in
       Arg.(
@@ -825,7 +884,7 @@ module Builder = struct
             ~docs
             ~docv:"FILE"
             ~env:(Cmd.Env.info ~doc "DUNE_BUILD_DIR")
-            ~doc)
+            ~doc:(Some doc))
     and+ diff_command =
       let doc =
         "Shell command to use to diff files. Use - to disable printing the diff."
@@ -833,7 +892,11 @@ module Builder = struct
       Arg.(
         value
         & opt (some string) None
-        & info [ "diff-command" ] ~docs ~env:(Cmd.Env.info ~doc "DUNE_DIFF_COMMAND") ~doc)
+        & info
+            [ "diff-command" ]
+            ~docs
+            ~env:(Cmd.Env.info ~doc "DUNE_DIFF_COMMAND")
+            ~doc:(Some doc))
     and+ stats_trace_file =
       Arg.(
         value
@@ -843,15 +906,9 @@ module Builder = struct
             ~docs
             ~docv:"FILE"
             ~doc:
-              "Output trace data in catapult format (compatible with chrome://tracing).")
-    and+ stats_trace_extended =
-      Arg.(
-        value
-        & flag
-        & info
-            [ "trace-extended" ]
-            ~docs
-            ~doc:"Output extended trace data (requires trace-file).")
+              (Some
+                 "Output trace data in catapult format (compatible with \
+                  chrome://tracing)."))
     and+ no_print_directory =
       Arg.(
         value
@@ -859,7 +916,7 @@ module Builder = struct
         & info
             [ "no-print-directory" ]
             ~docs
-            ~doc:"Suppress \"Entering directory\" messages.")
+            ~doc:(Some "Suppress \"Entering directory\" messages."))
     and+ store_orig_src_dir =
       let doc = "Store original source location in dune-package metadata." in
       Arg.(
@@ -869,7 +926,7 @@ module Builder = struct
             [ "store-orig-source-dir" ]
             ~docs
             ~env:(Cmd.Env.info ~doc "DUNE_STORE_ORIG_SOURCE_DIR")
-            ~doc)
+            ~doc:(Some doc))
     and+ () = build_info
     and+ instrument_with =
       let doc =
@@ -885,7 +942,7 @@ module Builder = struct
             ~docs
             ~env:(Cmd.Env.info ~doc "DUNE_INSTRUMENT_WITH")
             ~docv:"BACKENDS"
-            ~doc)
+            ~doc:(Some doc))
     and+ file_watcher =
       let doc =
         "Mechanism to detect changes in the source. Automatic to make dune run an \
@@ -898,7 +955,7 @@ module Builder = struct
             (enum
                [ "automatic", Dune_engine.Scheduler.Run.Automatic; "manual", No_watcher ])
             Automatic
-        & info [ "file-watcher" ] ~doc)
+        & info [ "file-watcher" ] ~doc:(Some doc))
     and+ watch_exclusions =
       let std_exclusions = Dune_config.standard_watch_exclusions in
       let doc =
@@ -915,7 +972,7 @@ module Builder = struct
         Arg.(
           value
           & opt_all (list ~sep:';' string) [ std_exclusions ]
-          & info [ "watch-exclusions" ] ~docs ~docv:"REGEX" ~doc)
+          & info [ "watch-exclusions" ] ~docs ~docv:"REGEX" ~doc:(Some doc))
       in
       Term.(const List.flatten $ arg)
     and+ wait_for_filesystem_clock =
@@ -925,13 +982,14 @@ module Builder = struct
         & info
             [ "wait-for-filesystem-clock" ]
             ~doc:
-              "Dune digest file contents for better incrementally. These digests are \
-               themselves cached. In some cases, Dune needs to drop some digest cache \
-               entries in order for things to be reliable. This option makes Dune wait \
-               for the file system clock to advance so that it doesn't need to drop \
-               anything. You should probably not care about this option; it is mostly \
-               useful for Dune developers to make Dune tests of the digest cache more \
-               reproducible.")
+              (Some
+                 "Dune digest file contents for better incrementally. These digests are \
+                  themselves cached. In some cases, Dune needs to drop some digest cache \
+                  entries in order for things to be reliable. This option makes Dune \
+                  wait for the file system clock to advance so that it doesn't need to \
+                  drop anything. You should probably not care about this option; it is \
+                  mostly useful for Dune developers to make Dune tests of the digest \
+                  cache more reproducible."))
     and+ cache_debug_flags = cache_debug_flags_term
     and+ report_errors_config =
       Arg.(
@@ -946,28 +1004,27 @@ module Builder = struct
         & info
             [ "error-reporting" ]
             ~doc:
-              "Controls when the build errors are reported. $(b,early) reports errors as \
-               soon as they are discovered. $(b,deterministic) reports errors at the end \
-               of the build in a deterministic order. $(b,twice) reports each error \
-               twice: once as soon as the error is discovered and then again at the end \
-               of the build, in a deterministic order.")
+              (Some
+                 "Controls when the build errors are reported. $(b,early) reports errors \
+                  as soon as they are discovered. $(b,deterministic) reports errors at \
+                  the end of the build in a deterministic order. $(b,twice) reports each \
+                  error twice: once as soon as the error is discovered and then again at \
+                  the end of the build, in a deterministic order."))
     and+ separate_error_messages =
       Arg.(
         value
         & flag
         & info
             [ "display-separate-messages" ]
-            ~doc:"Separate error messages with a blank line.")
+            ~doc:(Some "Separate error messages with a blank line."))
     and+ stop_on_first_error =
       Arg.(
         value
         & flag
         & info
             [ "stop-on-first-error" ]
-            ~doc:"Stop the build as soon as an error is encountered.")
+            ~doc:(Some "Stop the build as soon as an error is encountered."))
     in
-    if Option.is_none stats_trace_file && stats_trace_extended
-    then User_error.raise [ Pp.text "--trace-extended can only be used with --trace" ];
     { debug_dep_path
     ; debug_backtraces
     ; debug_artifact_substitution
@@ -1017,12 +1074,97 @@ module Builder = struct
     ; build_dir = Option.value ~default:default_build_dir build_dir
     ; root
     ; stats_trace_file
-    ; stats_trace_extended
     ; allow_builds = true
     ; default_root_is_cwd = false
     ; log_file = Default
     }
   ;;
+
+  let default = Term.eval_no_args_empty_env term
+
+  let equal
+        t
+        { debug_dep_path
+        ; debug_backtraces
+        ; debug_artifact_substitution
+        ; debug_load_dir
+        ; debug_digests
+        ; debug_package_logs
+        ; wait_for_filesystem_clock
+        ; only_packages
+        ; capture_outputs
+        ; diff_command
+        ; promote
+        ; ignore_promoted_rules
+        ; force
+        ; no_print_directory
+        ; ignore_lock_dir
+        ; store_orig_src_dir
+        ; default_target
+        ; watch
+        ; print_metrics
+        ; dump_memo_graph_file
+        ; dump_memo_graph_format
+        ; dump_memo_graph_with_timing
+        ; dump_gc_stats
+        ; always_show_command_line
+        ; promote_install_files
+        ; file_watcher
+        ; workspace_config
+        ; cache_debug_flags
+        ; report_errors_config
+        ; separate_error_messages
+        ; stop_on_first_error
+        ; require_dune_project_file
+        ; watch_exclusions
+        ; build_dir
+        ; root
+        ; stats_trace_file
+        ; allow_builds
+        ; default_root_is_cwd
+        ; log_file
+        }
+    =
+    Bool.equal t.debug_dep_path debug_dep_path
+    && Bool.equal t.debug_backtraces debug_backtraces
+    && Bool.equal t.debug_artifact_substitution debug_artifact_substitution
+    && Bool.equal t.debug_load_dir debug_load_dir
+    && Bool.equal t.debug_digests debug_digests
+    && Bool.equal t.debug_package_logs debug_package_logs
+    && Bool.equal t.wait_for_filesystem_clock wait_for_filesystem_clock
+    && Only_packages.Clflags.equal t.only_packages only_packages
+    && Bool.equal t.capture_outputs capture_outputs
+    && Option.equal String.equal t.diff_command diff_command
+    && Option.equal Dune_engine.Clflags.Promote.equal t.promote promote
+    && Bool.equal t.ignore_promoted_rules ignore_promoted_rules
+    && Bool.equal t.force force
+    && Bool.equal t.no_print_directory no_print_directory
+    && Bool.equal t.ignore_lock_dir ignore_lock_dir
+    && Bool.equal t.store_orig_src_dir store_orig_src_dir
+    && Arg.Dep.equal t.default_target default_target
+    && Dune_rpc_impl.Watch_mode_config.equal t.watch watch
+    && Bool.equal t.print_metrics print_metrics
+    && Option.equal Path.External.equal t.dump_memo_graph_file dump_memo_graph_file
+    && Graph.File_format.equal t.dump_memo_graph_format dump_memo_graph_format
+    && Bool.equal t.dump_memo_graph_with_timing dump_memo_graph_with_timing
+    && Option.equal Path.External.equal t.dump_gc_stats dump_gc_stats
+    && Bool.equal t.always_show_command_line always_show_command_line
+    && Bool.equal t.promote_install_files promote_install_files
+    && Dune_engine.Scheduler.Run.file_watcher_equal t.file_watcher file_watcher
+    && Source.Workspace.Clflags.equal t.workspace_config workspace_config
+    && Dune_engine.Cache_debug_flags.equal t.cache_debug_flags cache_debug_flags
+    && Dune_engine.Report_errors_config.equal t.report_errors_config report_errors_config
+    && Bool.equal t.separate_error_messages separate_error_messages
+    && Bool.equal t.stop_on_first_error stop_on_first_error
+    && Bool.equal t.require_dune_project_file require_dune_project_file
+    && List.equal String.equal t.watch_exclusions watch_exclusions
+    && String.equal t.build_dir build_dir
+    && Option.equal String.equal t.root root
+    && Option.equal String.equal t.stats_trace_file stats_trace_file
+    && Bool.equal t.allow_builds allow_builds
+    && Bool.equal t.default_root_is_cwd default_root_is_cwd
+    && Log.File.equal t.log_file log_file
+  ;;
 end
 
 type t =
@@ -1030,7 +1172,7 @@ type t =
   ; root : Workspace_root.t
   ; rpc :
       [ `Allow of Dune_lang.Dep_conf.t Dune_rpc_impl.Server.t Lazy.t | `Forbid_builds ]
-  ; stats : Dune_stats.t option
+  ; stats : Dune_trace.t option
   }
 
 let capture_outputs t = t.builder.capture_outputs
@@ -1119,20 +1261,11 @@ let print_entering_message c =
 
 (* CR-someday rleshchinskiy: The split between `build` and `init` seems quite arbitrary,
    we should probably refactor that at some point. *)
-let build (builder : Builder.t) =
-  let root =
-    Workspace_root.create_exn
-      ~default_is_cwd:builder.default_root_is_cwd
-      ~specified_by_user:builder.root
-  in
+let build (root : Workspace_root.t) (builder : Builder.t) =
   let stats =
     Option.map builder.stats_trace_file ~f:(fun f ->
-      let stats =
-        Dune_stats.create
-          ~extended_build_job_info:builder.stats_trace_extended
-          (Out (open_out f))
-      in
-      Dune_stats.set_global stats;
+      let stats = Dune_trace.create (Out (open_out f)) in
+      Dune_trace.set_global stats;
       stats)
   in
   let rpc =
@@ -1155,7 +1288,7 @@ let build (builder : Builder.t) =
              ~registry
              ~root:root.dir
              ~handle:Dune_rules_rpc.register
-             ~parse_build:Dune_rules_rpc.parse_build
+             ~parse_build_arg:Dune_rules_rpc.parse_build_arg
              stats))
     else `Forbid_builds
   in
@@ -1180,8 +1313,8 @@ let maybe_init_cache (cache_config : Dune_cache.Config.t) =
        Disabled)
 ;;
 
-let init (builder : Builder.t) =
-  let c = build builder in
+let init_with_root ~(root : Workspace_root.t) (builder : Builder.t) =
+  let c = build root builder in
   if c.root.dir <> Filename.current_dir_name then Sys.chdir c.root.dir;
   Path.set_root (normalize_path (Path.External.cwd ()));
   Path.Build.set_build_dir (Path.Outside_build_dir.of_string c.builder.build_dir);
@@ -1191,12 +1324,10 @@ let init (builder : Builder.t) =
   (* We need to print this before reading the workspace file, so that the editor
      can interpret errors in the workspace file. *)
   print_entering_message c;
-  Dune_rules.Workspace.Clflags.set c.builder.workspace_config;
+  Workspace.Clflags.set c.builder.workspace_config;
   let config =
     (* Here we make the assumption that this computation doesn't yield. *)
-    Fiber.run
-      (Memo.run (Dune_rules.Workspace.workspace_config ()))
-      ~iter:(fun () -> assert false)
+    Fiber.run (Memo.run (Workspace.workspace_config ())) ~iter:(fun () -> assert false)
   in
   let config =
     Dune_config.adapt_display
@@ -1211,9 +1342,9 @@ let init (builder : Builder.t) =
        | Yes _ -> true);
   Dune_engine.Execution_parameters.init
     (let open Memo.O in
-     let+ w = Dune_rules.Workspace.workspace () in
+     let+ w = Workspace.workspace () in
      Dune_engine.Execution_parameters.builtin_default
-     |> Dune_rules.Workspace.update_execution_parameters w);
+     |> Workspace.update_execution_parameters w);
   Dune_rules.Global.init ~capture_outputs:c.builder.capture_outputs;
   let cache_config =
     match config.cache_enabled with
@@ -1232,7 +1363,7 @@ let init (builder : Builder.t) =
   Log.info
     [ Pp.textf
         "Shared cache location: %s"
-        (Path.to_string Dune_cache_storage.Layout.root_dir)
+        (Path.to_string (Lazy.force Dune_cache_storage.Layout.root_dir))
     ];
   Dune_rules.Main.init
     ~stats:c.stats
@@ -1273,15 +1404,12 @@ let init (builder : Builder.t) =
     ];
   Dune_console.separate_messages c.builder.separate_error_messages;
   Option.iter c.stats ~f:(fun stats ->
-    if Dune_stats.extended_build_job_info stats
-    then
-      (* Communicate config settings as an instant event here. *)
-      let open Chrome_trace in
-      let args = [ "build_dir", `String (Path.Build.to_string Path.Build.root) ] in
-      let ts = Event.Timestamp.of_float_seconds (Unix.gettimeofday ()) in
-      let common = Event.common_fields ~cat:[ "config" ] ~name:"config" ~ts () in
-      let event = Event.instant ~args common in
-      Dune_stats.emit stats event);
+    let event =
+      Dune_trace.Event.config
+        ~version:
+          (Build_info.V1.version () |> Option.map ~f:Build_info.V1.Version.to_string)
+    in
+    Dune_trace.emit stats event);
   (* Setup hook for printing GC stats to a file *)
   at_exit (fun () ->
     match c.builder.dump_gc_stats with
@@ -1295,6 +1423,17 @@ let init (builder : Builder.t) =
   c, config
 ;;
 
+let init (builder : Builder.t) =
+  let root =
+    Workspace_root.create_exn
+      ~from:Filename.current_dir_name
+      ~default_is_cwd:builder.default_root_is_cwd
+      ~specified_by_user:builder.root
+      ()
+  in
+  init_with_root ~root builder
+;;
+
 let footer =
   `Blocks [ `S "BUGS"; `P "Check bug reports at https://github.com/ocaml/dune/issues" ]
 ;;
