@@ -21,18 +21,21 @@ let term =
   let+ default_builder = Common.Builder.term
   and+ context =
     let doc = "Run the Coq toplevel in this build context." in
-    Common.context_arg ~doc
+    Common.context_arg ~doc:(Some doc)
   and+ coqtop =
     let doc = "Run the given toplevel command instead of the default." in
-    Arg.(value & opt string "coqtop" & info [ "toplevel" ] ~docv:"CMD" ~doc)
+    Arg.(value & opt string "coqtop" & info [ "toplevel" ] ~docv:"CMD" ~doc:(Some doc))
   and+ coq_file_arg =
-    Arg.(required & pos 0 (some string) None (Arg.info [] ~docv:"COQFILE"))
-  and+ extra_args = Arg.(value & pos_right 0 string [] (Arg.info [] ~docv:"ARGS"))
+    (* CR-someday Alizter: document this option *)
+    Arg.(required & pos 0 (some string) None (Arg.info [] ~docv:"COQFILE" ~doc:None))
+  and+ extra_args =
+    (* CR-someday Alizter: document this option *)
+    Arg.(value & pos_right 0 string [] (Arg.info [] ~docv:"ARGS" ~doc:None))
   and+ no_rebuild =
     Arg.(
       value
       & flag
-      & info [ "no-build" ] ~doc:"Don't rebuild dependencies before executing.")
+      & info [ "no-build" ] ~doc:(Some "Don't rebuild dependencies before executing."))
   in
   let common, config =
     let builder =
@@ -41,8 +44,8 @@ let term =
     Common.init builder
   in
   let coq_file_arg = Common.prefix_target common coq_file_arg |> Path.Local.of_string in
-  let coqtop, argv, env =
-    Scheduler.go ~common ~config
+  let coqtop, args, env =
+    Scheduler.go_with_rpc_server ~common ~config
     @@ fun () ->
     let open Fiber.O in
     let* setup = Import.Main.setup () in
@@ -59,7 +62,7 @@ let term =
       |> Path.Build.append_local (Context.build_dir context)
     in
     let* coqtop, args, env =
-      Build_system.run_exn
+      build_exn
       @@ fun () ->
       let open Memo.O in
       let* (tr : Dune_rules.Dir_contents.triage) =
@@ -147,13 +150,13 @@ let term =
       let+ env = Super_context.context_env sctx in
       Path.to_string prog, args, env
     in
-    let argv =
+    let args =
       let topfile = Path.to_absolute_filename (Path.build coq_file_build) in
-      (coqtop :: "-topfile" :: topfile :: args) @ extra_args
+      ("-topfile" :: topfile :: args) @ extra_args
     in
-    Fiber.return (coqtop, argv, env)
+    Fiber.return (coqtop, args, env)
   in
-  restore_cwd_and_execve common coqtop argv env
+  restore_cwd_and_execve (Common.root common) coqtop args env
 ;;
 
 let command = Cmd.v info term
