@@ -21,6 +21,8 @@ let subcommands =
   ; Describe_pkg.command
   ; Describe_contexts.command
   ; Describe_depexts.command
+  ; Describe_location.command
+  ; Describe_tests.command
   ]
 ;;
 
