@@ -85,21 +85,19 @@ let resolve_lib db name kind =
 
 let resolve_lib_pps db preprocess =
   let open Memo.O in
-  let* pps =
-    Resolve.Memo.read_memo
-      (Dune_rules.Preprocess.Per_module.with_instrumentation
-         preprocess
-         ~instrumentation_backend:(Dune_rules.Lib.DB.instrumentation_backend db))
-    >>| Dune_rules.Preprocess.Per_module.pps
-  in
-  Memo.parallel_map ~f:(fun (_, name) -> resolve_lib db name Kind.Required) pps
+  Dune_rules.Instrumentation.with_instrumentation
+    preprocess
+    ~instrumentation_backend:(Dune_rules.Lib.DB.instrumentation_backend db)
+  |> Resolve.Memo.read_memo
+  >>| Dune_lang.Preprocess.Per_module.pps
+  >>= Memo.parallel_map ~f:(fun (_, name) -> resolve_lib db name Kind.Required)
 ;;
 
 let resolve_lib_deps db lib_deps =
   let open Memo.O in
   Memo.parallel_map lib_deps ~f:(fun (lib : Lib_dep.t) ->
     match lib with
-    | Direct (_, name) | Re_export (_, name) ->
+    | Direct (_, name) | Re_export (_, name) | Instantiate { lib = name; _ } ->
       let+ v = resolve_lib db name Kind.Required in
       [ v ]
     | Select select ->
@@ -203,17 +201,17 @@ let to_dyn context_name external_resolved_libs =
 
 let term =
   let+ builder = Common.Builder.term
-  and+ context_name = Common.context_arg ~doc:"Build context to use."
+  and+ context_name = Common.context_arg ~doc:(Some "Build context to use.")
   and+ _ = Describe_lang_compat.arg
   and+ format = Describe_format.arg in
   let common, config = Common.init builder in
-  Scheduler.go ~common ~config
+  Scheduler.go_with_rpc_server ~common ~config
   @@ fun () ->
   let open Fiber.O in
   let* setup = Import.Main.setup () in
   let* setup = Memo.run setup in
   let super_context = Import.Main.find_scontext_exn setup ~name:context_name in
-  Build_system.run_exn
+  build_exn
   @@ fun () ->
   let open Memo.O in
   let context_name =
