@@ -8,7 +8,7 @@ let all = [ "sexp", Sexp; "csexp", Csexp ]
 
 let arg =
   let doc = Printf.sprintf "$(docv) must be %s" (Arg.doc_alts_enum all) in
-  Arg.(value & opt (enum all) Sexp & info [ "format" ] ~docv:"FORMAT" ~doc)
+  Arg.(value & opt (enum all) Sexp & info [ "format" ] ~docv:"FORMAT" ~doc:(Some doc))
 ;;
 
 let print_as_sexp dyn =
@@ -23,7 +23,7 @@ let print_as_sexp dyn =
     |> Dune_lang.Ast.add_loc ~loc:Loc.none
     |> Dune_lang.Cst.concrete
   in
-  let version = Dune_lang.Syntax.greatest_supported_version Stanza.syntax in
+  let version = Dune_lang.Syntax.greatest_supported_version_exn Stanza.syntax in
   Pp.to_fmt Stdlib.Format.std_formatter (Dune_lang.Format.pp_top_sexps ~version [ cst ])
 ;;
 
