@@ -19,7 +19,7 @@ module Options = struct
     & flag
     & info
         [ "with-deps" ]
-        ~doc:"Whether the dependencies between modules should be printed."
+        ~doc:(Some "Whether the dependencies between modules should be printed.")
   ;;
 
   let arg_with_pps =
@@ -29,8 +29,9 @@ module Options = struct
     & info
         [ "with-pps" ]
         ~doc:
-          "Whether the dependencies towards ppx-rewriters (that are called at compile \
-           time) should be taken into account."
+          (Some
+             "Whether the dependencies towards ppx-rewriters (that are called at compile \
+              time) should be taken into account.")
   ;;
 
   let arg_sanitize_for_tests =
@@ -40,8 +41,9 @@ module Options = struct
     & info
         [ "sanitize-for-tests" ]
         ~doc:
-          "Sanitize the absolute paths in workspace items, and the associated UIDs, so \
-           that the output is reproducible."
+          (Some
+             "Sanitize the absolute paths in workspace items, and the associated UIDs, \
+              so that the output is reproducible.")
   ;;
 
   let arg : t Term.t =
@@ -71,9 +73,9 @@ module Descr = struct
   (* Description of the dependencies of a module *)
   module Mod_deps = struct
     type t =
-      { for_intf : Dune_rules.Module_name.t list
+      { for_intf : Dune_lang.Module_name.t list
         (* direct module dependencies for the interface *)
-      ; for_impl : Dune_rules.Module_name.t list
+      ; for_impl : Dune_lang.Module_name.t list
         (* direct module dependencies for the implementation *)
       }
 
@@ -81,8 +83,8 @@ module Descr = struct
     let to_dyn { for_intf; for_impl } =
       let open Dyn in
       record
-        [ "for_intf", list Dune_rules.Module_name.to_dyn for_intf
-        ; "for_impl", list Dune_rules.Module_name.to_dyn for_impl
+        [ "for_intf", list Dune_lang.Module_name.to_dyn for_intf
+        ; "for_impl", list Dune_lang.Module_name.to_dyn for_impl
         ]
     ;;
   end
@@ -90,7 +92,7 @@ module Descr = struct
   (* Description of modules *)
   module Mod = struct
     type t =
-      { name : Dune_rules.Module_name.t (* name of the module *)
+      { name : Dune_lang.Module_name.t (* name of the module *)
       ; impl : Path.t option (* path to the .ml file, if any *)
       ; intf : Path.t option (* path to the .mli file, if any *)
       ; cmt : Path.t option (* path to the .cmt file, if any *)
@@ -114,7 +116,7 @@ module Descr = struct
         | Some module_deps -> [ module_deps ]
       in
       record
-      @@ [ "name", Dune_rules.Module_name.to_dyn name
+      @@ [ "name", Dune_lang.Module_name.to_dyn name
          ; "impl", option dyn_path impl
          ; "intf", option dyn_path intf
          ; "cmt", option dyn_path cmt
@@ -245,7 +247,7 @@ module Lang = struct
            & info
                [ "lang" ]
                ~docv:"VERSION"
-               ~doc:"Behave the same as this version of Dune.")
+               ~doc:(Some "Behave the same as this version of Dune."))
        in
        if v = (0, 1)
        then `Ok v
@@ -336,14 +338,14 @@ module Crawl = struct
   let immediate_deps_of_module ~options ~obj_dir ~modules unit =
     match (options : Options.t) with
     | { with_deps = false; _ } ->
-      Action_builder.return { Ocaml.Ml_kind.Dict.intf = []; impl = [] }
+      Action_builder.return { Root.Ocaml.Ml_kind.Dict.intf = []; impl = [] }
     | { with_deps = true; _ } ->
       let deps ml_kind =
-        Dune_rules.Dep_rules.immediate_deps_of unit modules ~obj_dir ~ml_kind
+        Dune_rules.Dep_rules.read_immediate_deps_of ~obj_dir ~modules ~ml_kind unit
       in
       let open Action_builder.O in
       let+ intf, impl = Action_builder.both (deps Intf) (deps Impl) in
-      { Ocaml.Ml_kind.Dict.intf; impl }
+      { Root.Ocaml.Ml_kind.Dict.intf; impl }
   ;;
 
   (* Builds the description of a module from a module and its object directory *)
@@ -377,7 +379,7 @@ module Crawl = struct
     |> Modules.fold ~init:(Memo.return []) ~f:(fun m macc ->
       let* acc = macc in
       let deps = deps_of m in
-      let+ { Ocaml.Ml_kind.Dict.intf = deps_for_intf; impl = deps_for_impl }, _ =
+      let+ { Root.Ocaml.Ml_kind.Dict.intf = deps_for_intf; impl = deps_for_impl }, _ =
         Dune_engine.Action_builder.evaluate_and_collect_facts deps
       in
       module_ ~obj_dir ~deps_for_intf ~deps_for_impl m :: acc)
@@ -413,7 +415,8 @@ module Crawl = struct
         in
         Staged.unstage
         @@ Pp_spec.pped_modules_map
-             (Preprocess.Per_module.without_instrumentation exes.buildable.preprocess)
+             (Dune_lang.Preprocess.Per_module.without_instrumentation
+                exes.buildable.preprocess)
              version
       in
       let deps_of module_ =
@@ -481,7 +484,7 @@ module Crawl = struct
             in
             Staged.unstage
             @@ Pp_spec.pped_modules_map
-                 (Preprocess.Per_module.without_instrumentation
+                 (Dune_lang.Preprocess.Per_module.without_instrumentation
                     (Lib_info.preprocess info))
                  version
           in
@@ -624,9 +627,11 @@ let term : unit Term.t =
           []
           ~docv:"DIRS"
           ~doc:
-            "prints a description of the workspace's structure. If some directories DIRS \
-             are provided, then only those directories of the workspace are considered.")
-  and+ context_name = Common.context_arg ~doc:"Build context to use."
+            (Some
+               "prints a description of the workspace's structure. If some directories \
+                DIRS are provided, then only those directories of the workspace are \
+                considered."))
+  and+ context_name = Common.context_arg ~doc:(Some "Build context to use.")
   and+ format = Describe_format.arg
   and+ lang = Lang.arg
   and+ options = Options.arg in
@@ -655,7 +660,7 @@ let term : unit Term.t =
     in
     Dune_lang.Decoder.parse parse Univ_map.empty ast
   in
-  Scheduler.go ~common ~config
+  Scheduler.go_with_rpc_server ~common ~config
   @@ fun () ->
   let open Fiber.O in
   let* setup = Import.Main.setup () in
