@@ -8,13 +8,13 @@ module Cst = Dune_lang.Cst
 (** Abstractions around the kinds of files handled during initialization *)
 module File = struct
   type dune =
-    { path : Path.t
+    { dir : Path.Source.t
     ; name : string
     ; content : Cst.t list
     }
 
   type text =
-    { path : Path.t
+    { dir : Path.Source.t
     ; name : string
     ; content : string
     }
@@ -23,10 +23,10 @@ module File = struct
     | Dune of dune
     | Text of text
 
-  let make_text path name content = Text { path; name; content }
+  let make_text ~dir name content = Text { dir; name; content }
 
   let full_path = function
-    | Dune { path; name; _ } | Text { path; name; _ } -> Path.relative path name
+    | Dune { dir; name; _ } | Text { dir; name; _ } -> Path.Source.relative dir name
   ;;
 
   (** Inspection and manipulation of stanzas in a file *)
@@ -59,8 +59,11 @@ module File = struct
       | _ -> false
     ;;
 
-    let csts_conflict project (a : Cst.t) (b : Cst.t) =
-      let of_ast = Dune_rules.Stanzas.of_ast project in
+    let csts_conflict project ~dir (a : Cst.t) (b : Cst.t) =
+      let of_ast sexp =
+        let parser = Dune_project.stanza_parser project ~dir in
+        Dune_lang.Decoder.parse parser Univ_map.empty sexp
+      in
       (let open Option.O in
        let* a_ast = Cst.abstract a in
        let+ b_ast = Cst.abstract b in
@@ -71,19 +74,19 @@ module File = struct
     ;;
 
     (* TODO(shonfeder): replace with stanza merging *)
-    let find_conflicting project new_stanzas existing_stanzas =
+    let find_conflicting project ~dir new_stanzas existing_stanzas =
       let conflicting_stanza stanza =
-        match List.find ~f:(csts_conflict project stanza) existing_stanzas with
+        match List.find ~f:(csts_conflict ~dir project stanza) existing_stanzas with
         | Some conflict -> Some (stanza, conflict)
         | None -> None
       in
       List.find_map ~f:conflicting_stanza new_stanzas
     ;;
 
-    let add (project : Dune_project.t) stanzas = function
+    let add (project : Dune_project.t) ~dir stanzas = function
       | Text f -> Text f (* Adding a stanza to a text file isn't meaningful *)
       | Dune f ->
-        (match find_conflicting project stanzas f.content with
+        (match find_conflicting project ~dir stanzas f.content with
          | None -> Dune { f with content = f.content @ stanzas }
          | Some (a, b) ->
            User_error.raise
@@ -102,6 +105,7 @@ module File = struct
   (* Stanza *)
 
   let create_dir path =
+    let path = Path.source path in
     try Path.mkdir_p path with
     | Unix.Unix_error (EACCES, _, _) ->
       User_error.raise
@@ -112,9 +116,9 @@ module File = struct
         ]
   ;;
 
-  let load_dune_file ~path =
+  let load_dune_file ~dir =
     let name = "dune" in
-    let full_path = Path.relative path name in
+    let full_path = Path.relative (Path.source dir) name in
     let content =
       if not (Path.exists full_path)
       then []
@@ -135,18 +139,18 @@ module File = struct
                 (Path.to_string_maybe_quoted full_path)
             ])
     in
-    Dune { path; name; content }
+    Dune { dir; name; content }
   ;;
 
   let write_dune_file (dune_file : dune) =
-    let path = Path.relative dune_file.path dune_file.name in
+    let path = Path.Source.relative dune_file.dir dune_file.name in
     let version =
       Dune_lang.Syntax.greatest_supported_version_exn Dune_lang.Stanza.syntax
     in
     Io.with_file_out
       ~binary:true
       (* Why do we pass [~binary:true] but not anywhere else when formatting? *)
-      path
+      (Path.source path)
       ~f:(fun oc ->
         let fmt = Format.formatter_of_out_channel oc in
         Format.fprintf
@@ -161,6 +165,7 @@ module File = struct
     match f with
     | Dune f -> Ok (write_dune_file f)
     | Text f ->
+      let path = Path.source path in
       if Path.exists path
       then Error path
       else Ok (Io.write_file ~binary:false path f.content)
@@ -172,7 +177,7 @@ module Init_context = struct
   open Dune_config_file
 
   type t =
-    { dir : Path.t
+    { dir : Path.Source.t
     ; project : Dune_project.t
     ; defaults : Dune_config.Project_defaults.t
     }
@@ -180,11 +185,12 @@ module Init_context = struct
   let make path defaults =
     let open Memo.O in
     let+ project =
-      (* CR-rgrinberg: why not get the project from the source tree? *)
+      (* CR-someday rgrinberg: why not get the project from the source tree? *)
       Dune_project.load
         ~dir:Path.Source.root
         ~files:Filename.Set.empty
         ~infer_from_opam_files:true
+        ~load_opam_file_with_contents:Dune_pkg.Opam_file.load_opam_file_with_contents
       >>| function
       | Some p -> p
       | None ->
@@ -195,8 +201,20 @@ module Init_context = struct
     in
     let dir =
       match path with
-      | None -> Path.root
-      | Some p -> Path.of_string p
+      | None -> Path.Source.root
+      | Some p ->
+        (match Path.Outside_build_dir.of_string p with
+         | In_source_dir s -> s
+         | External e ->
+           (match Path.Expert.try_localize_external (Path.external_ e) with
+            | In_build_dir _ ->
+              (* Impossible because we never passed in a build path. It would
+                 be nice to have a [Path.Outside_build_dir.try_localize_external]
+                 reflect that
+              *)
+              assert false
+            | In_source_tree p -> p
+            | External _ -> User_error.raise [ Pp.textf "%s isn't in the workspace" p ]))
     in
     File.create_dir dir;
     { dir; project; defaults }
@@ -205,8 +223,8 @@ end
 
 let check_module_name name =
   let s = Dune_lang.Atom.to_string name in
-  let (_ : Dune_rules.Module_name.t) =
-    Dune_rules.Module_name.of_string_user_error (Loc.none, s) |> User_error.ok_exn
+  let (_ : Dune_lang.Module_name.t) =
+    Dune_lang.Module_name.of_string_user_error (Loc.none, s) |> User_error.ok_exn
   in
   ()
 ;;
@@ -319,7 +337,7 @@ module Component = struct
 
   (** Internal representation of the files comprising a component *)
   type target =
-    { dir : Path.t
+    { dir : Path.Source.t
     ; files : File.t list
     }
 
@@ -415,6 +433,7 @@ module Component = struct
                 ; constraint_ = None
                 }
               ]
+            ~contents_basename:None
         in
         let packages = Package.Name.Map.singleton (Package.name package) package in
         let info =
@@ -442,7 +461,7 @@ module Component = struct
 
   (* TODO Support for merging in changes to an existing stanza *)
   let add_stanza_to_dune_file ~(project : Dune_project.t) ~dir stanza =
-    File.load_dune_file ~path:dir |> File.Stanza.add project stanza
+    File.load_dune_file ~dir |> File.Stanza.add ~dir project stanza
   ;;
 
   (* Functions to make the various components, represented as lists of files *)
@@ -451,12 +470,12 @@ module Component = struct
       let dir = context.dir in
       let bin_dune =
         Stanza_cst.executable common options
-        |> add_stanza_to_dune_file ~project:context.project ~dir
+        |> add_stanza_to_dune_file ~dir ~project:context.project
       in
       let bin_ml =
         let name = sprintf "%s.ml" (Dune_lang.Atom.to_string common.name) in
         let content = sprintf "let () = print_endline \"Hello, World!\"\n" in
-        File.make_text dir name content
+        File.make_text ~dir name content
       in
       let files = [ bin_dune; bin_ml ] in
       [ { dir; files } ]
@@ -473,7 +492,6 @@ module Component = struct
     ;;
 
     let test ({ context; common; options } : Options.Test.t Options.t) =
-      (* Marking the current absence of test-specific options *)
       let dir = context.dir in
       let test_dune =
         Stanza_cst.test common options
@@ -482,13 +500,15 @@ module Component = struct
       let test_ml =
         let name = sprintf "%s.ml" (Dune_lang.Atom.to_string common.name) in
         let content = "" in
-        File.make_text dir name content
+        File.make_text ~dir name content
       in
       let files = [ test_dune; test_ml ] in
       [ { dir; files } ]
     ;;
 
-    let dune_project_file dir ({ context; common; options } : Options.Project.t Options.t)
+    let dune_project_file
+          ~dir
+          ({ context; common; options } : Options.Project.t Options.t)
       =
       let opam_file_gen =
         match options.pkg with
@@ -499,33 +519,48 @@ module Component = struct
         Stanza_cst.dune_project
           ~opam_file_gen
           ~defaults:context.defaults
-          Path.(as_in_source_tree_exn context.dir)
+          context.dir
           common
       in
-      File.Dune { path = dir; content; name = "dune-project" }
+      File.Dune { dir; content; name = "dune-project" }
+    ;;
+
+    (* Convert a libname to a dune atom that can be used to declare a library as
+       a dependency  *)
+    let lib_name_to_atom lib : Dune_lang.Atom.t =
+      Lib_name.to_string lib
+      |> String.map ~f:(function
+        | '-' -> '_' (* in case the lib_name is public, containing a `-` *)
+        | c -> c)
+      |> Dune_lang.Atom.of_string
     ;;
 
     let proj_exec dir ({ context; common; options } : Options.Project.t Options.t) =
       let lib_target =
         src
-          { context = { context with dir = Path.relative dir "lib" }
+          { context = { context with dir = Path.Source.relative dir "lib" }
           ; options = { inline_tests = options.inline_tests }
           ; common = { common with public = None }
           }
       in
       let test_target =
         let test_name = "test_" ^ Dune_lang.Atom.to_string common.name in
+        let libraries =
+          match common.public with
+          | None -> []
+          | Some lib -> [ lib_name_to_atom lib ]
+        in
         test
-          { context = { context with dir = Path.relative dir "test" }
+          { context = { context with dir = Path.Source.relative dir "test" }
           ; options = ()
-          ; common = { common with name = Dune_lang.Atom.of_string test_name }
+          ; common = { common with name = Dune_lang.Atom.of_string test_name; libraries }
           }
       in
       let bin_target =
         (* Add the lib_target as a library to the executable*)
         let libraries = Stanza_cst.add_to_list_set common.name common.libraries in
         bin
-          { context = { context with dir = Path.relative dir "bin" }
+          { context = { context with dir = Path.Source.relative dir "bin" }
           ; options = ()
           ; common = { common with libraries; name = Dune_lang.Atom.of_string "main" }
           }
@@ -536,17 +571,22 @@ module Component = struct
     let proj_lib dir ({ context; common; options } : Options.Project.t Options.t) =
       let lib_target =
         src
-          { context = { context with dir = Path.relative dir "lib" }
+          { context = { context with dir = Path.Source.relative dir "lib" }
           ; options = { inline_tests = options.inline_tests }
           ; common
           }
       in
       let test_target =
         let test_name = "test_" ^ Dune_lang.Atom.to_string common.name in
+        let libraries =
+          match common.public with
+          | None -> []
+          | Some lib -> [ lib_name_to_atom lib ]
+        in
         test
-          { context = { context with dir = Path.relative dir "test" }
+          { context = { context with dir = Path.Source.relative dir "test" }
           ; options = ()
-          ; common = { common with name = Dune_lang.Atom.of_string test_name }
+          ; common = { common with name = Dune_lang.Atom.of_string test_name; libraries }
           }
       in
       lib_target @ test_target
@@ -560,18 +600,21 @@ module Component = struct
           match (pkg : Options.Project.Pkg.t) with
           | Opam ->
             let name = Options.Common.package_name common in
-            let opam_file = Path.source @@ Package_name.file name ~dir in
-            [ File.make_text (Path.parent_exn opam_file) (Path.basename opam_file) "" ]
-          | Esy -> [ File.make_text (Path.source dir) "package.json" "" ]
+            let opam_file = Package_name.file name ~dir in
+            [ File.make_text
+                ~dir:(Path.Source.parent_exn opam_file)
+                (Path.Source.basename opam_file)
+                ""
+            ]
+          | Esy -> [ File.make_text ~dir "package.json" "" ]
         in
-        let dir = Path.source dir in
-        { dir; files = dune_project_file dir opts :: package_files }
+        { dir; files = dune_project_file ~dir opts :: package_files }
       in
       let component_targets =
         (match (template : Options.Project.Template.t) with
          | Exec -> proj_exec
          | Lib -> proj_lib)
-          (Path.source dir)
+          dir
           opts
       in
       proj_target :: component_targets
