@@ -70,7 +70,8 @@ let print_completion kind name =
 
 let path =
   let docv = "PATH" in
-  Arg.(value & pos 1 (some string) None & info [] ~docv)
+  (* CR-someday Alizter: document this option *)
+  Arg.(value & pos 1 (some string) None & info [] ~docv ~doc:None)
 ;;
 
 let context_cwd : Init_context.t Term.t =
@@ -79,7 +80,7 @@ let context_cwd : Init_context.t Term.t =
   let builder = Common.Builder.set_default_root_is_cwd builder true in
   let common, config = Common.init builder in
   let project_defaults = config.project_defaults in
-  Scheduler.go ~common ~config (fun () ->
+  Scheduler.go_with_rpc_server ~common ~config (fun () ->
     Memo.run (Init_context.make path project_defaults))
 ;;
 
@@ -118,13 +119,13 @@ end
 let libraries =
   let docv = "LIBRARIES" in
   let doc = "A comma separated list of libraries on which the component depends" in
-  Arg.(value & opt (list atom_conv) [] & info [ "libs" ] ~docv ~doc)
+  Arg.(value & opt (list atom_conv) [] & info [ "libs" ] ~docv ~doc:(Some doc))
 ;;
 
 let pps =
   let docv = "PREPROCESSORS" in
   let doc = "A comma separated list of ppx preprocessors used by the component" in
-  Arg.(value & opt (list atom_conv) [] & info [ "ppx" ] ~docv ~doc)
+  Arg.(value & opt (list atom_conv) [] & info [ "ppx" ] ~docv ~doc:(Some doc))
 ;;
 
 let public : Public_name.t option Term.t =
@@ -136,13 +137,14 @@ let public : Public_name.t option Term.t =
   Arg.(
     value
     & opt ~vopt:(Some Public_name.Use_name) (some Public_name.conv) None
-    & info [ "public" ] ~docv ~doc)
+    & info [ "public" ] ~docv ~doc:(Some doc))
 ;;
 
 let common : Component.Options.Common.t Term.t =
   let+ name =
     let docv = "NAME" in
-    Arg.(required & pos 0 (some component_name_conv) None & info [] ~docv)
+    (* CR-someday Alizter: document this option *)
+    Arg.(required & pos 0 (some component_name_conv) None & info [] ~docv ~doc:None)
   and+ public = public
   and+ libraries = libraries
   and+ pps = pps in
@@ -153,7 +155,8 @@ let common : Component.Options.Common.t Term.t =
 let project_common : Component.Options.Common.t Term.t =
   let+ project_name =
     let docv = "NAME" in
-    Arg.(required & pos 0 (some project_name_conv) None & info [] ~docv)
+    (* CR-someday Alizter: document this option *)
+    Arg.(required & pos 0 (some project_name_conv) None & info [] ~docv ~doc:None)
   and+ libraries = libraries
   and+ pps = pps in
   let public = Dune_project_name.to_string_hum project_name in
@@ -177,7 +180,7 @@ let inline_tests : bool Term.t =
     "Whether to use inline tests. Only applicable for $(b,library) and $(b,project) \
      components."
   in
-  Arg.(value & flag & info [ "inline-tests" ] ~docv ~doc)
+  Arg.(value & flag & info [ "inline-tests" ] ~docv ~doc:(Some doc))
 ;;
 
 let opt_default ~default term = Term.(const (Option.value ~default) $ term)
@@ -245,7 +248,7 @@ let project =
          Arg.(
            value
            & opt (some (enum Component.Options.Project.Template.commands)) None
-           & info [ "kind" ] ~docv ~doc)
+           & info [ "kind" ] ~docv ~doc:(Some doc))
      and+ pkg =
        let docv = "PACKAGE_MANAGER" in
        let doc =
@@ -257,7 +260,7 @@ let project =
          Arg.(
            value
            & opt (some (enum Component.Options.Project.Pkg.commands)) None
-           & info [ "pkg" ] ~docv ~doc)
+           & info [ "pkg" ] ~docv ~doc:(Some doc))
      in
      let name =
        match common.public with
@@ -279,7 +282,8 @@ let project =
        let (_ : Fpath.mkdir_p_result) = Fpath.mkdir_p root in
        let common, config = Common.init builder in
        let project_defaults = config.project_defaults in
-       Scheduler.go ~common ~config (fun () -> Memo.run @@ init_context project_defaults)
+       Scheduler.go_with_rpc_server ~common ~config (fun () ->
+         Memo.run @@ init_context project_defaults)
      in
      Component.init
        (Project { context; common; options = { template; inline_tests; pkg } });
