@@ -1,4 +1,3 @@
-open Stdune
 open Import
 module Persistent = Dune_util.Persistent
 
@@ -10,18 +9,19 @@ let man =
       {|Dump the contents of a file stored in Dune's persistent database in a human readable format.|}
   ; `Blocks Common.help_secs
   ]
+;;
 
-let info = Term.info "dump" ~doc ~man
+let info = Cmd.info "dump" ~doc ~man
 
 let term =
-  let+ common = Common.term
+  let+ builder = Common.Builder.term
   and+ file =
-    Arg.(required & pos 0 (some Arg.path) None & Arg.info [] ~docv:"FILE")
-  in
-  let _config = Common.init common in
-  let (Persistent.T ((module D), data)) =
-    Persistent.load_exn (Arg.Path.path file)
+    (* CR-someday Alizter: document this option *)
+    Arg.(required & pos 0 (some Arg.path) None & Arg.info [] ~docv:"FILE" ~doc:None)
   in
+  let _common, _config = Common.init builder in
+  let (Persistent.T ((module D), data)) = Persistent.load_exn (Arg.Path.path file) in
   Console.print [ Dyn.pp (D.to_dyn data) ]
+;;
 
-let command = (term, info)
+let command = Cmd.v info term
