@@ -25,7 +25,9 @@ let trim_broken_metadata_entries ~trimmed_so_far =
     Version.Metadata.all
     ~init:trimmed_so_far
     ~f:(fun trimmed_so_far version ->
-      let metadata_entries = Layout.Versioned.list_metadata_entries version in
+      let metadata_entries =
+        Lazy.force (Layout.Versioned.list_metadata_entries version)
+      in
       let file_path =
         Layout.Versioned.file_path (Version.Metadata.file_version version)
       in
@@ -51,7 +53,7 @@ let trim_broken_metadata_entries ~trimmed_so_far =
                | Metadata.Artifacts { entries; _ } ->
                  List.exists entries ~f:(function
                    | { Artifacts.Metadata_entry.digest = Some file_digest; path = _ } ->
-                     let reference = file_path ~file_digest in
+                     let reference = Lazy.force (file_path ~file_digest) in
                      not (Path.exists reference)
                      (* no digest means it's a directory. *)
                    | { digest = None; path = _ } -> false))
@@ -79,7 +81,7 @@ let garbage_collect () =
 
 let files_in_cache_for_all_supported_versions () =
   List.concat_map Version.File.all ~f:(fun file_version ->
-    Layout.Versioned.list_file_entries file_version)
+    Lazy.force (Layout.Versioned.list_file_entries file_version))
 ;;
 
 (* We call a cached file "unused" if there are currently no hard links to it
