@@ -1,7 +1,6 @@
 module Dune_config = struct
   open Stdune
   open Dune_lang.Decoder
-  module Spawn = Dune_spawn.Spawn
   module Display = Display
   module Scheduler = Dune_engine.Scheduler
   module Sandbox_mode = Dune_engine.Sandbox_mode
@@ -9,9 +8,7 @@ module Dune_config = struct
   module Stanza = Dune_lang.Stanza
   module String_with_vars = Dune_lang.String_with_vars
   module Pform = Dune_lang.Pform
-  module Log = Dune_util.Log
   module Config = Dune_config.Config
-  module Execution_env = Dune_util.Execution_env
 
   (* the configuration file use the same version numbers as dune-project files for
      simplicity *)
@@ -25,6 +22,13 @@ module Dune_config = struct
       ; license : string list option
       }
 
+    let equal t { authors; maintainers; maintenance_intent; license } =
+      Option.equal (List.equal String.equal) t.authors authors
+      && Option.equal (List.equal String.equal) t.maintainers maintainers
+      && Option.equal (List.equal String.equal) t.maintenance_intent maintenance_intent
+      && Option.equal (List.equal String.equal) t.license license
+    ;;
+
     let decode =
       fields
         (let+ authors = field_o "authors" (repeat string)
@@ -46,6 +50,32 @@ module Dune_config = struct
     ;;
   end
 
+  module Pkg_enabled = struct
+    type t =
+      | Set of Loc.t * Dune_config.Config.Toggle.t
+      | Unset
+
+    let decode =
+      let open Dune_lang.Decoder in
+      let+ loc, value = located (enum [ "enabled", `Enabled; "disabled", `Disabled ]) in
+      Set (loc, value)
+    ;;
+
+    let equal x y =
+      match x, y with
+      | Set (x_loc, x_toggle), Set (y_loc, y_toggle) ->
+        Loc.equal x_loc y_loc && Dune_config.Config.Toggle.equal x_toggle y_toggle
+      | Set _, _ | _, Set _ -> false
+      | Unset, Unset -> true
+    ;;
+
+    let to_dyn = function
+      | Set (loc, toggle) ->
+        Dyn.variant "Set" [ Loc.to_dyn loc; Config.Toggle.to_dyn toggle ]
+      | Unset -> Dyn.variant "Unset" []
+    ;;
+  end
+
   module Terminal_persistence = struct
     type t =
       | Preserve
@@ -59,6 +89,14 @@ module Dune_config = struct
       ]
     ;;
 
+    let equal a b =
+      match a, b with
+      | Preserve, Preserve
+      | Clear_on_rebuild, Clear_on_rebuild
+      | Clear_on_rebuild_and_flush_history, Clear_on_rebuild_and_flush_history -> true
+      | _, _ -> false
+    ;;
+
     let to_dyn = function
       | Preserve -> Dyn.Variant ("Preserve", [])
       | Clear_on_rebuild -> Dyn.Variant ("Clear_on_rebuild", [])
@@ -74,6 +112,13 @@ module Dune_config = struct
       | Fixed of int
       | Auto
 
+    let equal a b =
+      match a, b with
+      | Fixed a, Fixed b -> Int.equal a b
+      | Auto, Auto -> true
+      | _, _ -> false
+    ;;
+
     let error = Error "invalid concurrency value, must be 'auto' or a positive number"
 
     let of_string = function
@@ -105,6 +150,8 @@ module Dune_config = struct
   module Sandboxing_preference = struct
     type t = Sandbox_mode.t list
 
+    let equal = List.equal Sandbox_mode.equal
+
     let decode : Sandbox_mode.t Dune_sexp.Decoder.t =
       let open Dune_sexp.Decoder in
       enum
@@ -125,6 +172,14 @@ module Dune_config = struct
         | Enabled_except_user_rules
         | Enabled
 
+      let equal a b =
+        match a, b with
+        | Disabled, Disabled
+        | Enabled_except_user_rules, Enabled_except_user_rules
+        | Enabled, Enabled -> true
+        | _, _ -> false
+      ;;
+
       let to_string = function
         | Disabled -> "disabled"
         | Enabled_except_user_rules -> "enabled-except-user-rules"
@@ -166,6 +221,8 @@ module Dune_config = struct
     module Storage_mode = struct
       type t = Dune_cache_storage.Mode.t option
 
+      let equal = Option.equal Dune_cache_storage.Mode.equal
+
       let all =
         ("auto", None)
         :: List.map ~f:(fun (name, mode) -> name, Some mode) Dune_cache_storage.Mode.all
@@ -202,10 +259,63 @@ module Dune_config = struct
       ; action_stdout_on_success : Action_output_on_success.t field
       ; action_stderr_on_success : Action_output_on_success.t field
       ; project_defaults : Project_defaults.t field
+      ; pkg_enabled : Pkg_enabled.t field
       ; experimental : (string * (Loc.t * string)) list field
       }
   end
 
+  module Make_equal
+      (M : S)
+      (Equal : sig
+         val field : ('a -> 'a -> bool) -> 'a M.field -> 'a M.field -> bool
+       end) =
+  struct
+    let field = Equal.field
+
+    let equal
+          (t : M.t)
+          { M.display
+          ; concurrency
+          ; terminal_persistence
+          ; sandboxing_preference
+          ; cache_enabled
+          ; cache_reproducibility_check
+          ; cache_storage_mode
+          ; action_stdout_on_success
+          ; action_stderr_on_success
+          ; project_defaults
+          ; pkg_enabled
+          ; experimental
+          }
+      =
+      field Display.equal t.display display
+      && field Concurrency.equal t.concurrency concurrency
+      && field Terminal_persistence.equal t.terminal_persistence terminal_persistence
+      && field Sandboxing_preference.equal t.sandboxing_preference sandboxing_preference
+      && field Cache.Toggle.equal t.cache_enabled cache_enabled
+      && field
+           Dune_cache.Config.Reproducibility_check.equal
+           t.cache_reproducibility_check
+           cache_reproducibility_check
+      && field Cache.Storage_mode.equal t.cache_storage_mode cache_storage_mode
+      && field
+           Action_output_on_success.equal
+           t.action_stdout_on_success
+           action_stdout_on_success
+      && field
+           Action_output_on_success.equal
+           t.action_stderr_on_success
+           action_stderr_on_success
+      && field Project_defaults.equal t.project_defaults project_defaults
+      && field
+           (List.equal
+              (Tuple.T2.equal String.equal (Tuple.T2.equal Loc.equal String.equal)))
+           t.experimental
+           experimental
+      && field Pkg_enabled.equal t.pkg_enabled pkg_enabled
+    ;;
+  end
+
   module Make_superpose
       (A : S)
       (B : S)
@@ -230,6 +340,7 @@ module Dune_config = struct
       ; action_stderr_on_success =
           field a.action_stderr_on_success b.action_stderr_on_success
       ; project_defaults = field a.project_defaults b.project_defaults
+      ; pkg_enabled = field a.pkg_enabled b.pkg_enabled
       ; experimental = field a.experimental b.experimental
       }
     ;;
@@ -254,6 +365,7 @@ module Dune_config = struct
           ; action_stdout_on_success
           ; action_stderr_on_success
           ; project_defaults
+          ; pkg_enabled
           ; experimental
           }
       =
@@ -274,6 +386,7 @@ module Dune_config = struct
         ; ( "action_stderr_on_success"
           , field Action_output_on_success.to_dyn action_stderr_on_success )
         ; "project_defaults", field Project_defaults.to_dyn project_defaults
+        ; "pkg_enabled", field Pkg_enabled.to_dyn pkg_enabled
         ; ( "experimental"
           , field Dyn.(list (pair string (fun (_, v) -> string v))) experimental )
         ]
@@ -298,6 +411,7 @@ module Dune_config = struct
       ; action_stdout_on_success = None
       ; action_stderr_on_success = None
       ; project_defaults = None
+      ; pkg_enabled = None
       ; experimental = None
       }
     ;;
@@ -318,6 +432,13 @@ module Dune_config = struct
         (struct
           let field f = Dyn.option f
         end)
+
+    include
+      Make_equal
+        (M)
+        (struct
+          let field f = Option.equal f
+        end)
   end
 
   include
@@ -333,6 +454,13 @@ module Dune_config = struct
         let field f = f
       end)
 
+  include
+    Make_equal
+      (M)
+      (struct
+        let field f = f
+      end)
+
   let standard_watch_exclusions =
     [ {|^_opam|}
     ; {|/_opam|}
@@ -352,7 +480,6 @@ module Dune_config = struct
   ;;
 
   let hash = Poly.hash
-  let equal a b = Poly.equal a b
 
   let default =
     { display = Simple { verbosity = Quiet; status_line = not Execution_env.inside_dune }
@@ -370,6 +497,7 @@ module Dune_config = struct
         ; maintenance_intent = None
         ; license = Some [ "LICENSE" ]
         }
+    ; pkg_enabled = Unset
     ; experimental = []
     }
   ;;
@@ -436,6 +564,7 @@ module Dune_config = struct
     and+ action_stderr_on_success =
       field_o "action_stderr_on_success" (3, 0) Action_output_on_success.decode
     and+ project_defaults = field_o "project_defaults" (3, 17) Project_defaults.decode
+    and+ pkg_enabled = field_o "pkg" (3, 20) Pkg_enabled.decode
     and+ experimental =
       field_o "experimental" (3, 8) (repeat (pair string (located string)))
     in
@@ -457,6 +586,7 @@ module Dune_config = struct
     ; action_stdout_on_success
     ; action_stderr_on_success
     ; project_defaults
+    ; pkg_enabled
     ; experimental
     }
   ;;
@@ -471,8 +601,9 @@ module Dune_config = struct
   let decode_fields_of_workspace_file = decode_generic ~min_dune_version:(3, 0)
 
   let user_config_file =
-    let config_dir = Xdg.config_dir (Lazy.force Dune_util.xdg) in
-    Path.relative (Path.of_filename_relative_to_initial_cwd config_dir) "dune/config"
+    lazy
+      (let config_dir = Xdg.config_dir (Lazy.force Dune_util.xdg) in
+       Path.relative (Path.of_filename_relative_to_initial_cwd config_dir) "dune/config")
   ;;
 
   include Dune_lang.Versioned_file.Make (struct
@@ -483,10 +614,13 @@ module Dune_config = struct
 
   let load_config_file p =
     load_exn p ~f:(fun lang ->
-      String_with_vars.set_decoding_env (Pform.Env.initial lang.version) decode)
+      String_with_vars.set_decoding_env
+        (Pform.Env.initial ~stanza:lang.version ~extensions:[])
+        decode)
   ;;
 
   let load_user_config_file () =
+    let user_config_file = Lazy.force user_config_file in
     if Path.exists user_config_file
     then load_config_file user_config_file
     else Partial.empty
