@@ -20,6 +20,7 @@ module Dune_config : sig
       | Fixed of int
       | Auto
 
+    val equal : t -> t -> bool
     val of_string : string -> (t, string) result
     val to_string : t -> string
   end
@@ -53,6 +54,21 @@ module Dune_config : sig
     end
   end
 
+  module Pkg_enabled : sig
+    (** Configuration for Dune's package management features.
+        
+        - [Set (loc, `Enabled)]: Package management is explicitly enabled. Forces package 
+          management to be active even if no lock directories are present.
+
+        - [Set (loc, `Disabled)]: Package management is explicitly disabled. Forces package 
+          management to be inactive even if lock directories are present.
+
+        - [Unset]: Package management enablement is not explicitly configured.  *)
+    type t =
+      | Set of Loc.t * Dune_config.Config.Toggle.t
+      | Unset
+  end
+
   module Terminal_persistence : sig
     type t =
       | Preserve
@@ -82,6 +98,7 @@ module Dune_config : sig
       ; action_stdout_on_success : Action_output_on_success.t field
       ; action_stderr_on_success : Action_output_on_success.t field
       ; project_defaults : Project_defaults.t field
+      ; pkg_enabled : Pkg_enabled.t field
       ; experimental : (string * (Loc.t * string)) list field
       }
   end
@@ -94,6 +111,7 @@ module Dune_config : sig
     val empty : t
     val superpose : t -> t -> t
     val to_dyn : t -> Dyn.t
+    val equal : t -> t -> bool
   end
 
   (** A standard list of watch exclusions *)
@@ -109,7 +127,7 @@ module Dune_config : sig
 
   val superpose : t -> Partial.t -> t
   val default : t
-  val user_config_file : Path.t
+  val user_config_file : Path.t Lazy.t
 
   (** We return a [Partial.t] here so that the result can easily be merged with
       other sources of configurations. *)
@@ -134,7 +152,7 @@ module Dune_config : sig
   val for_scheduler
     :  t
     -> watch_exclusions:string list
-    -> Dune_stats.t option
+    -> Dune_trace.t option
     -> print_ctrl_c_warning:bool
     -> Dune_engine.Scheduler.Config.t
 end
