@@ -27,9 +27,9 @@ module Gen_rules = struct
     ;;
 
     let create
-      ?(build_dir_only_sub_dirs = empty.build_dir_only_sub_dirs)
-      ?(directory_targets = empty.directory_targets)
-      rules
+          ?(build_dir_only_sub_dirs = empty.build_dir_only_sub_dirs)
+          ?(directory_targets = empty.directory_targets)
+          rules
       =
       { build_dir_only_sub_dirs; directory_targets; rules }
     ;;
@@ -71,14 +71,13 @@ type t =
       -> src:Path.Build.t
       -> dst:Path.Source.t
       -> unit Fiber.t
-  ; stats : Dune_stats.t option
+  ; stats : Dune_trace.t option
   ; cache_config : Dune_cache.Config.t
   ; cache_debug_flags : Cache_debug_flags.t
   ; implicit_default_alias : Path.Build.t -> unit Action_builder.t option Memo.t
-  ; execution_parameters : dir:Path.Build.t -> Execution_parameters.t Memo.t
+  ; execution_parameters :
+      Context_name.t -> dir:Path.Build.t -> Execution_parameters.t Memo.t
   ; source_tree : (module Source_tree)
-  ; action_runner : Action_exec.input -> Action_runner.t option
-  ; action_runners : unit -> Action_runner.t list
   ; shared_cache : (module Dune_cache.Shared.S)
   ; write_error_summary : Build_system_error.Set.t -> unit Fiber.t
   }
@@ -87,20 +86,18 @@ let t : t Fdecl.t = Fdecl.create Dyn.opaque
 let get () = Fdecl.get t
 
 let set
-  ~action_runner
-  ~action_runners
-  ~stats
-  ~contexts
-  ~promote_source
-  ~cache_config
-  ~cache_debug_flags
-  ~sandboxing_preference
-  ~rule_generator
-  ~implicit_default_alias
-  ~execution_parameters
-  ~source_tree
-  ~shared_cache
-  ~write_error_summary
+      ~stats
+      ~contexts
+      ~promote_source
+      ~cache_config
+      ~cache_debug_flags
+      ~sandboxing_preference
+      ~rule_generator
+      ~implicit_default_alias
+      ~execution_parameters
+      ~source_tree
+      ~shared_cache
+      ~write_error_summary
   =
   let contexts =
     Memo.lazy_ ~name:"Build_config.set" (fun () ->
@@ -110,11 +107,6 @@ let set
         contexts
         ~f:(fun ((ctx : Build_context.t), ctx_type) -> ctx.name, (ctx, ctx_type)))
   in
-  let () =
-    match (cache_config : Dune_cache.Config.t) with
-    | Disabled -> ()
-    | Enabled _ -> Dune_cache_storage.Layout.create_cache_directories ()
-  in
   Fdecl.set
     t
     { contexts
@@ -128,8 +120,6 @@ let set
     ; implicit_default_alias
     ; execution_parameters
     ; source_tree
-    ; action_runner
-    ; action_runners
     ; shared_cache
     ; write_error_summary
     }
