@@ -3,14 +3,12 @@ module Digest = Dune_digest.Digest
 module Cached_digest = Dune_digest.Cached_digest
 module Console = Dune_console
 module Metrics = Dune_metrics
-module Log = Dune_util.Log
+module Compound_user_error = Dune_rpc_private.Compound_user_error
 module Stringlike = Dune_util.Stringlike
 
 module type Stringlike = Dune_util.Stringlike
 
-module Spawn = Dune_spawn.Spawn
 module Persistent = Dune_util.Persistent
-module Execution_env = Dune_util.Execution_env
 module Glob = Dune_glob.V1
 module Targets = Dune_targets
 include No_io
