@@ -17,5 +17,11 @@ val coqdep_flags : t -> Ordered_set_lang.Unexpanded.t
 (** Flags for coqdoc *)
 val coqdoc_flags : t -> Ordered_set_lang.Unexpanded.t
 
+(** Coqdoc header config. *)
+val coqdoc_header : t -> String_with_vars.t option
+
+(** Coqdoc footer config. *)
+val coqdoc_footer : t -> String_with_vars.t option
+
 (** Parser for env stanza. *)
 val decode : t Decoder.fields_parser
