@@ -1,5 +1,4 @@
-open Stdune
-open Dune_sexp
+open Import
 
 type part =
   | Text of string
@@ -102,7 +101,7 @@ let decode_manually f =
     ; loc
     ; parts =
         List.map parts ~f:(function
-          | Template.Text s -> Text s
+          | Template.Part.Text s -> Text s
           | Pform v ->
             (match f env v with
              | pform -> Pform (v, pform)
@@ -115,6 +114,11 @@ let decode_manually f =
 let decode = decode_manually Pform.Env.parse
 let loc t = t.loc
 
+let map_loc t ~f =
+  let loc = f t.loc in
+  { t with loc }
+;;
+
 let virt_pform ?quoted pos pform =
   let loc = Loc.of_pos pos in
   make_pform ?quoted loc pform
@@ -441,14 +445,14 @@ let has_pforms t = Option.is_none (text_only t)
 
 let encode t =
   match text_only t with
-  | Some s -> atom_or_quoted_string s
+  | Some s -> Dune_sexp.atom_or_quoted_string s
   | None ->
-    Template
+    Dune_sexp.Template
       { loc = t.loc
       ; quoted = t.quoted
       ; parts =
           List.map t.parts ~f:(function
-            | Text s -> Template.Text s
+            | Text s -> Template.Part.Text s
             | Error (_, msg) -> raise (User_error.E msg)
             | Pform (source, pform) ->
               (match Pform.encode_to_latest_dune_lang_version pform with
@@ -464,7 +468,7 @@ let encode t =
       }
 ;;
 
-let to_dyn t = to_dyn (encode t)
+let to_dyn t = Dune_sexp.to_dyn (encode t)
 
 let remove_locs { quoted; loc = _; parts } =
   { quoted
