@@ -1,12 +1,18 @@
-open! Import
-
 type t =
   | Ocamlformat
   | Odoc
   | Ocamllsp
+  | Utop
+  | Ocamlearlybird
+  | Odig
+  | Opam_publish
+  | Dune_release
+  | Ocaml_index
 
+val to_dyn : t -> Dyn.t
 val all : t list
 val equal : t -> t -> bool
+val hash : t -> int
 val package_name : t -> Package_name.t
 val of_package_name : Package_name.t -> t
 val exe_name : t -> string
