@@ -8,6 +8,7 @@ module Opam_solver = Opam_solver
 module OpamUrl = OpamUrl0
 module Package_variable = Package_variable
 module Package_dependency = Package_dependency
+module Dependency_formula = Dependency_formula
 module Rev_store = Rev_store
 module Solver_env = Solver_env
 module Solver_stats = Solver_stats
@@ -20,5 +21,9 @@ module Local_package = Local_package
 module Package_universe = Package_universe
 module Variable_value = Variable_value
 module Resolved_package = Resolved_package
-module Pin_stanza = Pin_stanza
+module Pin = Pin
 module Package_name = Package_name
+module Ocamlformat = Ocamlformat
+module Dev_tool = Dev_tool
+module Outdated = Outdated
+module Dune_dep = Dune_dep
