@@ -1,6 +1,5 @@
 include Stdune
 module Stringlike = Dune_util.Stringlike
-module Re = Dune_re
 
 module type Stringlike = Dune_util.Stringlike
 
@@ -18,6 +17,7 @@ include struct
   module Pform = Pform
   module Blang = Blang
   module Slang = Slang
+  module Package_variable_name = Package_variable_name
 end
 
 include struct
@@ -25,3 +25,5 @@ include struct
   module Process = Process
   module Display = Display
 end
+
+module OpamUrl = OpamUrl0
