@@ -1,3 +1,5 @@
+open Stdune
+
 (** Add values for expanding [%{name}] for a package *)
 val add_self_to_filter_env
   :  OpamPackage.t
@@ -14,4 +16,5 @@ val opam_package_to_lock_file_pkg
   -> OpamPackage.t
   -> pinned:bool
   -> Resolved_package.t
-  -> Lock_dir.Pkg.t
+  -> portable_lock_dir:bool
+  -> (Lock_dir.Pkg.t, User_message.t) result
