@@ -1,4 +1,3 @@
-open! Stdune
 include Dune_lang.Package_version
 
 let of_opam_package_version opam_version =
@@ -6,3 +5,4 @@ let of_opam_package_version opam_version =
 ;;
 
 let to_opam_package_version t = to_string t |> OpamPackage.Version.of_string
+let dev = of_string "dev"
