@@ -17,5 +17,5 @@ module Status : sig
   val sexp : (t, Conv.values) Conv.t
 end
 
-val build : (string list, Dune_rpc.Build_outcome_with_diagnostics.t) Decl.Request.t
+val build : (string list, Build_outcome_with_diagnostics.t) Decl.Request.t
 val status : (unit, Status.t) Decl.Request.t
