@@ -26,7 +26,7 @@ include struct
 end
 
 module Run = struct
-  module Registry = Dune_rpc_private.Registry
+  module Registry = Dune_rpc.Registry
 
   module Server = Dune_rpc_server.Make (struct
       include Csexp_rpc.Session
@@ -41,7 +41,7 @@ module Run = struct
     ; root : string
     ; where : Dune_rpc.Where.t
     ; server : Csexp_rpc.Server.t Lazy.t
-    ; stats : Dune_stats.t option
+    ; stats : Dune_trace.t option
     ; server_ivar : Csexp_rpc.Server.t Fiber.Ivar.t
     ; registry : [ `Add | `Skip ]
     }
@@ -107,8 +107,14 @@ module Run = struct
   ;;
 end
 
-type 'build_arg pending_build_action =
-  | Build of 'build_arg list * Dune_engine.Scheduler.Run.Build_outcome.t Fiber.Ivar.t
+type 'build_arg pending_action_kind =
+  | Build of 'build_arg list
+  | Runtest of string list
+
+type 'build_arg pending_action =
+  { kind : 'build_arg pending_action_kind
+  ; outcome : Dune_engine.Scheduler.Run.Build_outcome.t Fiber.Ivar.t
+  }
 
 module Client = Stdune.Unit
 
@@ -191,7 +197,7 @@ end
 
 type 'build_arg t =
   { config : Run.t
-  ; pending_build_jobs : ('build_arg list * Build_outcome.t Fiber.Ivar.t) Job_queue.t
+  ; pending_jobs : 'build_arg pending_action Job_queue.t
   ; parse_build_arg : string -> 'build_arg
   ; mutable clients : Clients.t
   }
@@ -230,9 +236,7 @@ let handler (t : _ t Fdecl.t) handle : 'build_arg Dune_rpc_server.Handler.t =
     t.clients <- Clients.remove_session t.clients session;
     Fiber.return ()
   in
-  let rpc =
-    Handler.create ~on_terminate ~on_init ~version:Dune_rpc_private.Version.latest ()
-  in
+  let rpc = Handler.create ~on_terminate ~on_init ~version:Dune_rpc.Version.latest () in
   let () =
     let module Error = Build_system_error in
     let diff ~last ~(now : Error.Set.t) =
@@ -328,25 +332,54 @@ let handler (t : _ t Fdecl.t) handle : 'build_arg Dune_rpc_server.Handler.t =
     Handler.declare_notification rpc Procedures.Server_side.log
   in
   let () = Handler.implement_request rpc Procedures.Public.ping (fun _ -> Fiber.return) in
-  let () =
-    let build _session targets =
+  let implement_request_pending_action decl ~f =
+    let handler _session input =
       let server = Fdecl.get t in
-      let ivar = Fiber.Ivar.create () in
-      let targets = List.map targets ~f:server.parse_build_arg in
-      let* () = Job_queue.write server.pending_build_jobs (targets, ivar) in
-      let+ build_outcome = Fiber.Ivar.read ivar in
+      let outcome = Fiber.Ivar.create () in
+      let* () = Job_queue.write server.pending_jobs { kind = f input; outcome } in
+      let+ build_outcome = Fiber.Ivar.read outcome in
       match (build_outcome : Build_outcome.t) with
-      | Success -> Dune_rpc_private.Build_outcome_with_diagnostics.Success
+      | Success -> Dune_rpc.Build_outcome_with_diagnostics.Success
       | Failure -> Failure (get_current_diagnostic_errors ())
     in
-    Handler.implement_request rpc Decl.build build
+    Handler.implement_request rpc decl handler
+  in
+  let () =
+    implement_request_pending_action Decl.build ~f:(fun targets ->
+      let server = Fdecl.get t in
+      let targets = List.map targets ~f:server.parse_build_arg in
+      Build targets)
+  in
+  let () =
+    implement_request_pending_action Procedures.Public.runtest ~f:(fun paths ->
+      Runtest paths)
+  in
+  let () =
+    let f _ () =
+      let server = Fdecl.get t in
+      let outcome = Fiber.Ivar.create () in
+      let target =
+        Dune_lang.Dep_conf.Alias_rec (Dune_lang.String_with_vars.make_text Loc.none "fmt")
+      in
+      let* () =
+        Job_queue.write server.pending_jobs { kind = Build [ target ]; outcome }
+      in
+      let+ build_outcome = Fiber.Ivar.read outcome in
+      match build_outcome with
+      (* A 'successful' formatting means there is nothing to promote. *)
+      | Success -> ()
+      | Failure ->
+        Promote.Diff_promotion.promote_files_registered_in_last_run
+          Dune_rpc.Files_to_promote.All
+    in
+    Handler.implement_request rpc Procedures.Public.format f
   in
   let () =
     let rec cancel_pending_jobs () =
-      match Job_queue.pop_internal (Fdecl.get t).pending_build_jobs with
+      match Job_queue.pop_internal (Fdecl.get t).pending_jobs with
       | None -> Fiber.return ()
-      | Some (_, job) ->
-        let* () = Fiber.Ivar.fill job Build_outcome.Failure in
+      | Some { kind = _; outcome } ->
+        let* () = Fiber.Ivar.fill outcome Build_outcome.Failure in
         cancel_pending_jobs ()
     in
     let shutdown _ () =
@@ -391,9 +424,9 @@ let handler (t : _ t Fdecl.t) handle : 'build_arg Dune_rpc_server.Handler.t =
   let () =
     let f _ files =
       Promote.Diff_promotion.promote_files_registered_in_last_run files;
-      Fiber.return Dune_rpc_private.Build_outcome_with_diagnostics.Success
+      Fiber.return Dune_rpc.Build_outcome_with_diagnostics.Success
     in
-    Handler.implement_request rpc Dune_rpc.Procedures.Public.promote_many f
+    Handler.implement_request rpc Procedures.Public.promote_many f
   in
   let () =
     let f _ path =
@@ -402,7 +435,7 @@ let handler (t : _ t Fdecl.t) handle : 'build_arg Dune_rpc_server.Handler.t =
         (These ([ files ], ignore));
       Fiber.return ()
     in
-    Handler.implement_request rpc Dune_rpc_private.Procedures.Public.promote f
+    Handler.implement_request rpc Procedures.Public.promote f
   in
   let () =
     let f _ () = Fiber.return Path.Build.(to_string root) in
@@ -414,7 +447,7 @@ let handler (t : _ t Fdecl.t) handle : 'build_arg Dune_rpc_server.Handler.t =
 
 let create ~lock_timeout ~registry ~root ~handle stats ~parse_build_arg =
   let t = Fdecl.create Dyn.opaque in
-  let pending_build_jobs = Job_queue.create () in
+  let pending_jobs = Job_queue.create () in
   let handler = Dune_rpc_server.make (handler t handle) in
   let pool = Fiber.Pool.create () in
   let where = Where.default () in
@@ -450,7 +483,7 @@ let create ~lock_timeout ~registry ~root ~handle stats ~parse_build_arg =
     ; server_ivar = Fiber.Ivar.create ()
     }
   in
-  let res = { config; pending_build_jobs; clients = Clients.empty; parse_build_arg } in
+  let res = { config; pending_jobs; clients = Clients.empty; parse_build_arg } in
   Fdecl.set t res;
   res
 ;;
@@ -460,7 +493,4 @@ let run t =
   Run.run t.config
 ;;
 
-let pending_build_action t =
-  Job_queue.read t.pending_build_jobs
-  |> Fiber.map ~f:(fun (targets, ivar) -> Build (targets, ivar))
-;;
+let pending_action t = Job_queue.read t.pending_jobs
