@@ -26,5 +26,7 @@ let install = standard "install"
 let pkg_install = Alias.Name.of_string "pkg-install"
 let ocaml_index = standard "ocaml-index"
 let runtest = standard "runtest"
+let unused_libs = standard "unused-libs"
 let all = standard "all"
 let default = standard "default"
+let empty = standard "empty"
