@@ -10,5 +10,6 @@ val gen_rules
   :  Super_context.t
   -> Executables.t
   -> dir:Path.Build.t
-  -> requires_link:Lib.t list Resolve.t Memo.Lazy.t
+  -> Lib.Compile.t
+  -> Dir_contents.t
   -> unit Memo.t
