@@ -1,4 +1,4 @@
-open! Import
+open Import
 
 (* This file is licensed under The MIT License *)
 (* (c) MINES ParisTech 2018-2019               *)
