@@ -9,7 +9,7 @@ let loc_of_dune_file st_dir =
      let open Option.O in
      let* dune_file = Source_tree.Dir.dune_file st_dir in
      (* TODO not really correct. we need to know the [(subdir ..)] that introduced this *)
-     Dune_file0.path dune_file
+     Source.Dune_file.path dune_file
    with
    | Some s -> s
    | None -> Path.Source.relative (Source_tree.Dir.path st_dir) "_unknown_")
@@ -22,9 +22,11 @@ type t =
   ; dir : Path.Build.t
   ; text_files : Filename.Set.t
   ; foreign_sources : Foreign_sources.t Memo.Lazy.t
-  ; mlds : (Documentation.t * Path.Build.t list) list Memo.Lazy.t
+  ; mlds : (Documentation.t * Doc_sources.mld list) list Memo.Lazy.t
   ; coq : Coq_sources.t Memo.Lazy.t
+  ; rocq : Rocq_sources.t Memo.Lazy.t
   ; ml : Ml_sources.t Memo.Lazy.t
+  ; source_dir : Source_tree.Dir.t option
   }
 
 and kind =
@@ -32,14 +34,16 @@ and kind =
   | Group_root of t list
   | Group_part
 
-let empty kind ~dir =
+let empty kind ~dir ~source_dir =
   { kind
   ; dir
+  ; source_dir
   ; text_files = Filename.Set.empty
   ; ml = Memo.Lazy.of_val Ml_sources.empty
   ; mlds = Memo.Lazy.of_val []
   ; foreign_sources = Memo.Lazy.of_val Foreign_sources.empty
   ; coq = Memo.Lazy.of_val Coq_sources.empty
+  ; rocq = Memo.Lazy.of_val Rocq_sources.empty
   }
 ;;
 
@@ -52,11 +56,11 @@ module Standalone_or_root = struct
 
   type nonrec t = { contents : standalone_or_root Memo.Lazy.t }
 
-  let empty ~dir =
+  let empty ~dir ~source_dir =
     { contents =
         Memo.Lazy.create (fun () ->
           Memo.return
-            { root = empty Standalone ~dir
+            { root = empty Standalone ~dir ~source_dir
             ; rules = Rules.empty
             ; subdirs = Path.Build.Map.empty
             })
@@ -84,9 +88,10 @@ type triage =
   | Group_part of Path.Build.t
 
 let dir t = t.dir
+let source_dir t = t.source_dir
 let coq t = Memo.Lazy.force t.coq
+let rocq t = Memo.Lazy.force t.rocq
 let ocaml t = Memo.Lazy.force t.ml
-let artifacts t = Memo.Lazy.force t.ml >>= Ml_sources.artifacts
 
 let dirs t =
   match t.kind with
@@ -118,40 +123,6 @@ let mlds t ~(stanza : Documentation.t) =
       ]
 ;;
 
-let build_mlds_map stanzas ~dir ~files =
-  let mlds =
-    Memo.lazy_ (fun () ->
-      Filename.Set.fold files ~init:Filename.Map.empty ~f:(fun fn acc ->
-        (* TODO this doesn't handle [.foo.mld] correctly *)
-        match String.lsplit2 fn ~on:'.' with
-        | Some (s, "mld") -> Filename.Map.set acc s fn
-        | _ -> acc)
-      |> Memo.return)
-  in
-  Dune_file.find_stanzas stanzas Documentation.key
-  >>= Memo.parallel_map ~f:(fun (doc : Documentation.t) ->
-    let+ mlds =
-      let+ mlds = Memo.Lazy.force mlds in
-      Ordered_set_lang.Unordered_string.eval
-        doc.mld_files
-        ~standard:mlds
-        ~key:Fun.id
-        ~parse:(fun ~loc s ->
-          match Filename.Map.find mlds s with
-          | Some s -> s
-          | None ->
-            User_error.raise
-              ~loc
-              [ Pp.textf
-                  "%s.mld doesn't exist in %s"
-                  s
-                  (Path.to_string_maybe_quoted
-                     (Path.drop_optional_build_context (Path.build dir)))
-              ])
-    in
-    doc, List.map (Filename.Map.values mlds) ~f:(Path.Build.relative dir))
-;;
-
 module rec Load : sig
   val get : Super_context.t -> dir:Path.Build.t -> t Memo.t
   val triage : Super_context.t -> dir:Path.Build.t -> triage Memo.t
@@ -161,7 +132,7 @@ end = struct
        dependencies *)
     List.filter_map libraries ~f:(fun dep ->
       match (dep : Lib_dep.t) with
-      | Re_export _ | Direct _ -> None
+      | Re_export _ | Direct _ | Instantiate _ -> None
       | Select s -> Some s.result_fn)
   ;;
 
@@ -183,6 +154,12 @@ end = struct
               Path.Build.set_extension mlg_file ~ext:".ml" |> Path.Build.basename)
           | Coq_stanza.Extraction.T s ->
             Memo.return (Coq_stanza.Extraction.ml_target_fnames s)
+          | Rocq_stanza.Rocqpp.T { modules; _ } ->
+            Rocq_sources.mlg_files ~sctx ~dir ~modules
+            >>| List.rev_map ~f:(fun mlg_file ->
+              Path.Build.set_extension mlg_file ~ext:".ml" |> Path.Build.basename)
+          | Rocq_stanza.Extraction.T s ->
+            Memo.return (Rocq_stanza.Extraction.ml_target_fnames s)
           | Menhir_stanza.T menhir -> Memo.return (Menhir_stanza.targets menhir)
           | Rule_conf.T rule ->
             Simple_rules.user_rule sctx rule ~dir ~expander
@@ -247,6 +224,12 @@ end = struct
       (Path.to_string_maybe_quoted (Path.build dir))
   ;;
 
+  let mlds ~sctx ~dir ~dune_file ~files =
+    Memo.lazy_ (fun () ->
+      let* expander = Super_context.expander sctx ~dir in
+      Doc_sources.build_mlds_map dune_file ~dir ~files expander)
+  ;;
+
   let make_standalone sctx st_dir ~dir (d : Dune_file.t) =
     let human_readable_description () = human_readable_description dir in
     { Standalone_or_root.contents =
@@ -264,7 +247,10 @@ end = struct
               let src_dir = Dune_file.dir d in
               stanzas >>= load_text_files sctx st_dir ~src_dir ~dir)
           in
-          let dirs = [ { Source_file_dir.dir; path_to_root = []; files } ] in
+          let dirs =
+            [ { Source_file_dir.dir; path_to_root = []; files; source_dir = Some st_dir }
+            ]
+          in
           let ml =
             Memo.lazy_ (fun () ->
               let lookup_vlib = lookup_vlib sctx ~current_dir:dir in
@@ -283,12 +269,14 @@ end = struct
                     ~lookup_vlib
                     ~dirs)
           in
+          let mlds = mlds ~sctx ~dir ~dune_file:d ~files in
           { Standalone_or_root.root =
               { kind = Standalone
+              ; source_dir = Some st_dir
               ; dir
               ; text_files = files
               ; ml
-              ; mlds = Memo.lazy_ (fun () -> build_mlds_map d ~dir ~files)
+              ; mlds
               ; foreign_sources =
                   Memo.lazy_ (fun () ->
                     let dune_version = Dune_project.dune_version project in
@@ -296,6 +284,9 @@ end = struct
               ; coq =
                   Memo.lazy_ (fun () ->
                     stanzas >>| Coq_sources.of_dir ~dir ~include_subdirs ~dirs)
+              ; rocq =
+                  Memo.lazy_ (fun () ->
+                    stanzas >>| Rocq_sources.of_dir ~dir ~include_subdirs ~dirs)
               }
           ; rules
           ; subdirs = Path.Build.Map.empty
@@ -343,9 +334,20 @@ end = struct
                             ~src_dir:(Source_tree.Dir.path source_dir)
                             ~dir
                         in
-                        { Source_file_dir.dir; path_to_root = path_to_group_root; files })))
+                        { Source_file_dir.dir
+                        ; path_to_root = path_to_group_root
+                        ; files
+                        ; source_dir = Some source_dir
+                        })))
+           in
+           let dirs =
+             { Source_file_dir.dir
+             ; path_to_root = []
+             ; files
+             ; source_dir = Some source_dir
+             }
+             :: subdirs
            in
-           let dirs = { Source_file_dir.dir; path_to_root = []; files } :: subdirs in
            let lib_config =
              let+ ocaml = Context.ocaml ctx in
              ocaml.lib_config
@@ -376,25 +378,36 @@ end = struct
              Memo.lazy_ (fun () ->
                stanzas >>| Coq_sources.of_dir ~dir ~dirs ~include_subdirs)
            in
+           let rocq =
+             Memo.lazy_ (fun () ->
+               stanzas >>| Rocq_sources.of_dir ~dir ~dirs ~include_subdirs)
+           in
+           let mlds = mlds ~sctx ~dir ~dune_file ~files in
            let subdirs =
-             List.map subdirs ~f:(fun { Source_file_dir.dir; path_to_root = _; files } ->
-               { kind = Group_part
-               ; dir
-               ; text_files = files
-               ; ml
-               ; foreign_sources
-               ; mlds = Memo.lazy_ (fun () -> build_mlds_map dune_file ~dir ~files)
-               ; coq
-               })
+             List.map
+               subdirs
+               ~f:(fun { Source_file_dir.dir; path_to_root = _; files; source_dir } ->
+                 { kind = Group_part
+                 ; source_dir
+                 ; dir
+                 ; text_files = files
+                 ; ml
+                 ; foreign_sources
+                 ; mlds
+                 ; coq
+                 ; rocq
+                 })
            in
            let root =
              { kind = Group_root subdirs
+             ; source_dir = Some source_dir
              ; dir
              ; text_files = files
              ; ml
              ; foreign_sources
-             ; mlds = Memo.lazy_ (fun () -> build_mlds_map dune_file ~dir ~files)
+             ; mlds
              ; coq
+             ; rocq
              }
            in
            { Standalone_or_root.root
@@ -410,8 +423,11 @@ end = struct
     >>= function
     | Is_component_of_a_group_but_not_the_root { group_root; stanzas = _ } ->
       Memo.return @@ Group_part group_root
-    | Lock_dir | Generated | Source_only _ ->
-      Memo.return @@ Standalone_or_root (Standalone_or_root.empty ~dir)
+    | Generated ->
+      Memo.return @@ Standalone_or_root (Standalone_or_root.empty ~dir ~source_dir:None)
+    | Lock_dir source_dir | Source_only source_dir ->
+      Memo.return
+      @@ Standalone_or_root (Standalone_or_root.empty ~dir ~source_dir:(Some source_dir))
     | Standalone (st_dir, d) ->
       Memo.return @@ Standalone_or_root (make_standalone sctx st_dir ~dir d)
     | Group_root root ->
@@ -474,9 +490,8 @@ let modules_of_lib sctx lib =
 
 let () =
   Fdecl.set Expander.lookup_artifacts (fun ~dir ->
-    Context.DB.by_dir dir
-    >>| Context.name
-    >>= Super_context.find_exn
-    >>= Load.get ~dir
-    >>= artifacts)
+    let* t =
+      Context.DB.by_dir dir >>| Context.name >>= Super_context.find_exn >>= Load.get ~dir
+    in
+    Memo.Lazy.force t.ml >>= Ml_sources.artifacts)
 ;;
