@@ -35,14 +35,13 @@ module Projects_and_dune_files =
       type t = status * Dune_project.t
     end))
     (Monoid.Appendable_list (struct
-         type t = Path.Source.t * Dune_project.t * Dune_file0.t
+         type t = Path.Source.t * Dune_project.t * Source.Dune_file.t
        end))
 
 module Source_tree_map_reduce =
   Source_tree.Make_map_reduce_with_progress (Memo) (Projects_and_dune_files)
 
 let load () =
-  let open Memo.O in
   let status dir =
     match Source_tree.Dir.status dir with
     | Vendored -> `Vendored
@@ -64,10 +63,13 @@ let load () =
       in
       Memo.return (projects, dune_files)
     in
-    Source_tree_map_reduce.map_reduce ~traverse:Source_dir_status.Set.all ~f
+    Source_tree_map_reduce.map_reduce
+      ~traverse:Source_dir_status.Set.all
+      ~trace_event_name:"Dune load"
+      ~f
   in
   let projects = Appendable_list.to_list_rev projects in
-  let packages, vendored_packages =
+  let all_packages, vendored_packages =
     List.fold_left
       projects
       ~init:(Package.Name.Map.empty, Package.Name.Set.empty)
@@ -91,12 +93,35 @@ let load () =
         in
         acc_packages, vendored)
   in
-  let mask = Only_packages.mask packages ~vendored:vendored_packages in
-  let packages = Only_packages.filter_packages mask packages in
+  let mask = Only_packages.mask all_packages ~vendored:vendored_packages in
+  let packages = Only_packages.filter_packages mask all_packages in
   let projects = List.rev_map projects ~f:snd in
   let dune_files =
     let without_ctx =
-      Memo.lazy_ ~name:"dune-files-eval" (fun () -> Dune_file.eval dune_files mask)
+      Memo.lazy_ ~name:"dune-files-eval" (fun () ->
+        let (_ : Package.Name.t Path.Source.Map.t) =
+          match
+            Package.Name.Map.values all_packages
+            |> List.filter_map ~f:(fun pkg ->
+              match Package.exclusive_dir pkg with
+              | None -> None
+              | Some d -> Some (d, pkg))
+            |> Path.Source.Map.of_list_map ~f:(fun ((_loc, d), pkg) ->
+              d, Package.name pkg)
+          with
+          | Ok s -> s
+          | Error (dir, ((loc, _), p1), (_, p2)) ->
+            let name p = Package.Name.to_string (Package.name p) in
+            User_error.raise
+              ~loc
+              [ Pp.textf
+                  "Directory %s cannot belong to package %s"
+                  (Path.Source.to_string_maybe_quoted dir)
+                  (name p1)
+              ; Pp.textf "It already belongs to package %s" (name p2)
+              ]
+        in
+        Dune_file.eval dune_files mask)
     in
     Per_context.create_by_name ~name:"dune-files" (fun ctx ->
       Memo.Lazy.create (fun () ->
@@ -124,7 +149,7 @@ let load =
 
 let find_project ~dir =
   let+ { projects_by_root; _ } = load () in
-  Find_closest_source_dir.find_by_dir projects_by_root ~dir
+  Find_closest_source_dir.find_by_dir_exn projects_by_root ~dir
 ;;
 
 let stanzas_in_dir dir =
