@@ -92,6 +92,7 @@ module Lib = struct
     let sub_systems = Lib_info.sub_systems info in
     let plugins = Lib_info.plugins info in
     let requires = Lib_info.requires info in
+    let parameters = Lib_info.parameters info in
     let foreign_objects =
       match Lib_info.foreign_objects info with
       | External e -> e
@@ -105,7 +106,6 @@ module Lib = struct
     let melange_runtime_deps = additional_paths (Lib_info.melange_runtime_deps info) in
     let jsoo_runtime = Lib_info.jsoo_runtime info in
     let wasmoo_runtime = Lib_info.wasmoo_runtime info in
-    let virtual_ = Lib_info.virtual_ info in
     let instrumentation_backend = Lib_info.instrumentation_backend info in
     let native_archives =
       match Lib_info.native_archives info with
@@ -124,7 +124,6 @@ module Lib = struct
     record_fields
     @@ [ field "name" Lib_name.encode name
        ; field "kind" Lib_kind.encode kind
-       ; field_b "virtual" virtual_
        ; field_o "synopsis" string synopsis
        ; field_o "orig_src_dir" path orig_src_dir
        ; mode_paths "archives" archives
@@ -140,6 +139,7 @@ module Lib = struct
        ; paths "jsoo_runtime" jsoo_runtime
        ; paths "wasmoo_runtime" wasmoo_runtime
        ; Lib_dep.L.field_encode requires ~name:"requires"
+       ; field_l "parameters" (no_loc Lib_name.encode) parameters
        ; libs "ppx_runtime_deps" ppx_runtime_deps
        ; field_o "implements" (no_loc Lib_name.encode) implements
        ; field_o "default_implementation" (no_loc Lib_name.encode) default_implementation
@@ -192,7 +192,20 @@ module Lib = struct
        let+ synopsis = field_o "synopsis" string
        and+ loc = loc
        and+ modes = field_l "modes" Lib_mode.decode
-       and+ kind = field "kind" Lib_kind.decode
+       and+ kind =
+         let* kind = field "kind" Lib_kind.decode in
+         let+ virtual_ = field_b "virtual" in
+         match kind with
+         | (Dune_file Normal | Virtual) when virtual_ ->
+           (* Backward compatible support for dune-project files
+              that include the [(virtual)] field. *)
+           Lib_kind.Virtual
+         | incompatible_kind when virtual_ ->
+           Code_error.raise
+             "invalid combination of 'kind' and 'virtual' fields in library stanza of \
+              dune-package file"
+             [ "kind", Lib_kind.to_dyn incompatible_kind; "virtual", Dyn.Bool virtual_ ]
+         | otherwise -> otherwise
        and+ archives = mode_paths "archives"
        and+ plugins = mode_paths "plugins"
        and+ foreign_objects = paths "foreign_objects"
@@ -217,8 +230,8 @@ module Lib = struct
        and+ wasmoo_runtime = paths "wasmoo_runtime"
        and+ melange_runtime_deps = paths "melange_runtime_deps"
        and+ requires = field_l "requires" (Lib_dep.decode ~allow_re_export:true)
+       and+ parameters = field "parameters" ~default:[] (repeat (located Lib_name.decode))
        and+ ppx_runtime_deps = libs "ppx_runtime_deps"
-       and+ virtual_ = field_b "virtual"
        and+ sub_systems = Sub_system_info.record_parser
        and+ orig_src_dir = field_o "orig_src_dir" path
        and+ modules = field "modules" (Modules.decode ~src_dir:base)
@@ -241,6 +254,7 @@ module Lib = struct
            | Public (_, _) -> Lib_info.Status.Installed
          in
          let version = None in
+         let local_main_module_name = main_module_name in
          let main_module_name = Lib_info.Inherited.This main_module_name in
          let foreign_objects = Lib_info.Source.External foreign_objects in
          let public_headers = Lib_info.File_deps.External public_headers in
@@ -268,13 +282,16 @@ module Lib = struct
            ~version
            ~synopsis
            ~main_module_name
+           ~local_main_module_name
            ~sub_systems
            ~requires
+           ~parameters
            ~foreign_objects
            ~public_headers
            ~plugins
            ~archives
            ~ppx_runtime_deps
+           ~allow_unused_libraries:[]
            ~foreign_archives
            ~native_archives:(Files native_archives)
            ~foreign_dll_files
@@ -284,7 +301,6 @@ module Lib = struct
            ~enabled
            ~virtual_deps
            ~dune_version
-           ~virtual_
            ~entry_modules
            ~implements
            ~default_implementation
@@ -295,6 +311,8 @@ module Lib = struct
            ~exit_module:None
            ~instrumentation_backend
            ~melange_runtime_deps
+           ~root_module:None
+         (* CR-someday rgrinberg: maybe we should add this to installed packages? *)
        in
        let external_location =
          let opam_dir = Path.parent_exn base in
@@ -592,11 +610,16 @@ module Or_meta = struct
 
   let parse file lexbuf =
     let dir = Path.parent_exn file in
+    let extensions = [ Dune_lang.Oxcaml.(syntax, latest_version) ] in
+    let with_extensions decoder =
+      List.fold_left extensions ~init:decoder ~f:(fun decoder (ext, version) ->
+        Syntax.set ext (Active version) decoder)
+    in
     match
       Vfile.parse_contents lexbuf ~f:(fun lang ->
         String_with_vars.set_decoding_env
-          (Pform.Env.initial lang.version)
-          (decode ~lang ~dir))
+          (Pform.Env.initial ~stanza:lang.version ~extensions)
+          (with_extensions (decode ~lang ~dir)))
     with
     | contents -> Ok contents
     | exception User_error.E message -> Error message
