@@ -668,6 +668,7 @@ let build_cm cctx ~dir ~in_context ~mode ~src ~obj_dir ~deps ~config:config_opt
         |> Action_builder.map ~f:Config.of_flags
       | Some config -> Action_builder.return config
     in
+    let deps = List.filter deps ~f:(fun m -> Module.has m ~ml_kind:Impl) in
     (Path.build (in_build_dir ctx ~config [ "stdlib"; with_js_ext ~mode "stdlib.cma" ])
      :: List.concat_map libs ~f:(fun lib -> jsoo_archives ~mode ctx config lib))
     @ List.map deps ~f:(fun m ->
@@ -855,10 +856,14 @@ let build_exe
           (Path.Build.set_extension src ~ext:(Js_of_ocaml.Ext.runtime ~mode))
       ]
   in
-  let rule_mode : Rule.Mode.t =
-    match promote with
-    | None -> Standard
-    | Some p -> Promote p
+  let* rule_mode : Rule.Mode.t =
+    let* expander = Super_context.expander sctx ~dir in
+    let rule_mode =
+      match promote with
+      | None -> Rule_mode.Standard
+      | Some p -> Promote p
+    in
+    Rule_mode_expand.expand_path ~expander ~dir rule_mode
   in
   let* cmode =
     match compilation_mode with
