@@ -11,8 +11,31 @@ val to_dyn : t -> Dyn.t
     or the [name] if not. *)
 val name : t -> Lib_name.t
 
-val lib_config : t -> Lib_config.t
 val implements : t -> t Resolve.Memo.t option
+val parameters : t -> t list Resolve.Memo.t
+
+module Parameterised : sig
+  type status =
+    | Not_parameterised
+    | Partial
+    | Complete
+
+  val status : t -> status
+  val arguments : t -> t list
+  val applied_modules : t -> Module_name.t Parameterised_name.t list Resolve.t
+  val applied_name : t -> Module_name.t Parameterised_name.t Resolve.t
+  val requires : t -> t list Resolve.t
+  val for_instance : build_dir:Path.Build.t -> ext_lib:string -> t -> t
+  val dir : build_dir:Path.Build.t -> t -> Path.Build.t
+
+  val instantiate
+    :  loc:Loc.t
+    -> from:[ `depends | `inline_tests ]
+    -> t
+    -> (Loc.t * t Resolve.t) list
+    -> parent_parameters:(Loc.t * t) list
+    -> t Resolve.t
+end
 
 (** [is_local t] returns [true] whenever [t] is defined in the local workspace *)
 val is_local : t -> bool
@@ -24,6 +47,7 @@ val wrapped : t -> Wrapped.t option Resolve.Memo.t
 (** Direct library dependencies of this library *)
 val requires : t -> t list Resolve.Memo.t
 
+val re_exports : t -> t list Resolve.Memo.t
 val ppx_runtime_deps : t -> t list Resolve.Memo.t
 val pps : t -> t list Resolve.Memo.t
 
@@ -41,6 +65,8 @@ module L : sig
     -> key:('a -> t)
     -> deps:('a -> 'a list Resolve.Memo.t)
     -> ('a list, 'a list) Result.t Resolve.Memo.t
+
+  val project_root : t list -> Path.Source.t option
 end
 
 (** {1 Compilation contexts} *)
@@ -60,6 +86,8 @@ module Compile : sig
   (** Return the list of dependencies needed for linking this library/exe *)
   val requires_link : t -> lib list Resolve.t Memo.Lazy.t
 
+  val user_written_requires : t -> (Loc.t * lib) list Resolve.Memo.t
+
   (** Dependencies listed by the user + runtime dependencies from ppx *)
   val direct_requires : t -> lib list Resolve.Memo.t
 
@@ -76,7 +104,8 @@ module Compile : sig
   (** Transitive closure of all used ppx rewriters *)
   val pps : t -> lib list Resolve.Memo.t
 
-  val merlin_ident : t -> Merlin_ident.t
+  (** Libraries allowed to be unused *)
+  val allow_unused_libraries : t -> lib list Resolve.Memo.t
 
   (** Sub-systems used in this compilation context *)
   val sub_systems : t -> sub_system list Memo.t
@@ -91,6 +120,13 @@ module DB : sig
   (** A database allow to resolve library names *)
   type t = db
 
+  val with_parent : t -> parent:t option -> t
+
+  (** Create a library database from a specified list of library paths. A
+      library path is a path to a "lib" directory such as those found in the
+      OCAMLPATH variable or the "path" field of findlib.conf. *)
+  val of_paths : Context.t -> paths:Path.t list -> t Memo.t
+
   val installed : Context.t -> t Memo.t
 
   module Resolve_result : sig
@@ -117,7 +153,6 @@ module DB : sig
     -> resolve:(Lib_name.t -> Resolve_result.t list Memo.t)
     -> resolve_lib_id:(Lib_id.t -> Resolve_result.t Memo.t)
     -> all:(unit -> Lib_name.t list Memo.t)
-    -> lib_config:Lib_config.t
     -> instrument_with:Lib_name.t list
     -> unit
     -> t
@@ -146,13 +181,13 @@ module DB : sig
       This function is for executables or melange.emit stanzas. *)
   val resolve_user_written_deps
     :  t
-    -> [ `Exe of (Import.Loc.t * string) list | `Melange_emit of string ]
+    -> [ `Exe of (Loc.t * string) Nonempty_list.t | `Melange_emit of string ]
     -> allow_overlaps:bool
     -> forbidden_libraries:(Loc.t * Lib_name.t) list
     -> Lib_dep.t list
+    -> allow_unused_libraries:(Loc.t * Lib_name.t) list
     -> pps:(Loc.t * Lib_name.t) list
     -> dune_version:Dune_lang.Syntax.Version.t
-    -> merlin_ident:Merlin_ident.t
     -> Compile.t
 
   val resolve_pps : t -> (Loc.t * Lib_name.t) list -> lib list Resolve.Memo.t
