@@ -1,5 +1,5 @@
 (** Link mode of OCaml programs *)
-open! Dune_engine
+open Import
 
 type t =
   | Byte
@@ -8,5 +8,4 @@ type t =
   | Byte_with_stubs_statically_linked_in
 
 val mode : t -> Mode.t
-
 val equal : t -> t -> bool
