@@ -26,33 +26,40 @@ let implicit_default_alias dir =
 ;;
 
 let execution_parameters =
-  let f path =
-    match Path.Build.drop_build_context path with
-    | None -> Dune_engine.Execution_parameters.default
-    | Some path ->
-      let open Memo.O in
-      let+ dir = Source_tree.nearest_dir path
-      and+ ep = Execution_parameters.default in
+  let f context path =
+    let path = Path.Build.drop_build_context_exn path in
+    let open Memo.O in
+    let* ep = Execution_parameters.default in
+    if Context_name.equal context Private_context.t.name
+    then Memo.return ep
+    else
+      let+ dir = Source_tree.nearest_dir path in
       Dune_project.update_execution_parameters (Source_tree.Dir.project dir) ep
   in
   let memo =
+    let module Input = struct
+      type t = Context_name.t * Path.Build.t
+
+      let hash = Tuple.T2.hash Context_name.hash Path.Build.hash
+      let equal = Tuple.T2.equal Context_name.equal Path.Build.equal
+      let to_dyn = Tuple.T2.to_dyn Context_name.to_dyn Path.Build.to_dyn
+    end
+    in
     Memo.create
       "execution-parameters-of-dir"
-      ~input:(module Path.Build)
+      ~input:(module Input)
       ~cutoff:Execution_parameters.equal
-      f
+      (fun (ctx, path) -> f ctx path)
   in
-  fun ~dir -> Memo.exec memo dir
+  fun context ~dir -> Memo.exec memo (context, dir)
 ;;
 
 let init
-  ?(action_runner = fun _ -> None)
-  ?(action_runners = fun _ -> [])
-  ~stats
-  ~sandboxing_preference
-  ~cache_config
-  ~(cache_debug_flags : Dune_engine.Cache_debug_flags.t)
-  ()
+      ~stats
+      ~sandboxing_preference
+      ~cache_config
+      ~(cache_debug_flags : Dune_engine.Cache_debug_flags.t)
+      ()
   : unit
   =
   let promote_source ~chmod ~delete_dst_if_it_is_a_directory ~src ~dst =
@@ -97,8 +104,6 @@ let init
     ~execution_parameters
     ~source_tree:(module Source_tree)
     ~shared_cache:(module Shared_cache)
-    ~action_runner
-    ~action_runners
     ~write_error_summary:(fun _ -> Fiber.return ())
 ;;
 
