@@ -1,5 +1,6 @@
 open Import
 open Memo.O
+module Parallel_map = Memo.Make_parallel_map (Module_name.Map)
 
 module Common = struct
   module Encode = struct
@@ -73,7 +74,7 @@ module Stdlib = struct
   let map t ~f = { t with modules = Module_name.Map.map t.modules ~f }
 
   let traverse t ~f =
-    let+ modules = Module_name.Parallel_map.parallel_map t.modules ~f:(fun _ -> f) in
+    let+ modules = Parallel_map.parallel_map t.modules ~f:(fun _ -> f) in
     { t with modules }
   ;;
 
@@ -184,16 +185,9 @@ module Mangle = struct
                |> Module_name.of_string
            ; public = main_module_name
            })
-    | Exe ->
-      sprintf "dune__exe"
-      |> Module_name.of_string
-      |> Visibility.Map.make_both
-      |> Option.some
+    | Exe -> Module_name.of_string "dune__exe" |> Visibility.Map.make_both |> Option.some
     | Melange ->
-      sprintf "melange"
-      |> Module_name.of_string
-      |> Visibility.Map.make_both
-      |> Option.some
+      Module_name.of_string "melange" |> Visibility.Map.make_both |> Option.some
     | Unwrapped -> None
   ;;
 
@@ -226,9 +220,8 @@ module Mangle = struct
       then None
       else
         Some
-          (Path.Local.L.relative
-             Path.Local.root
-             (List.map ~f:Module_name.uncapitalize path)
+          (List.map ~f:Module_name.uncapitalize path
+           |> Path.Local.L.relative Path.Local.root
            |> Path.Local.set_extension ~ext:".ml")
     in
     Module.generated ?install_as path ~obj_name ~kind ~src_dir:obj_dir
@@ -417,7 +410,9 @@ module Group = struct
     fun acc modules m -> loop acc modules (Module.path m)
   ;;
 
-  let parents (t : t) m = parents_modules [ t ] t.modules m
+  (* [parents acc modules m] returns [acc] followed by all parent groups of 
+     module [m], ordered from innermost to outermost parent. *)
+  let parents (t : t) m = parents_modules [ t ] t.modules m |> List.rev
 
   module Memo_traversals = struct
     let rec parallel_map ({ alias; modules; name = _ } as t) ~f =
@@ -427,7 +422,7 @@ module Group = struct
       { t with alias; modules }
 
     and parallel_map_modules modules ~f =
-      Module_name.Parallel_map.parallel_map modules ~f:(fun _ n ->
+      Parallel_map.parallel_map modules ~f:(fun _ n ->
         match n with
         | Module m ->
           let+ m = f m in
@@ -460,8 +455,11 @@ module Group = struct
         (* XXX ocamldep can't currently give us precise dependencies for
            modules under [(include_subdirs qualified)] directories. For that
            reason we currently depend on everything under the sub-directory. *)
-        Module_name.Map.values g.modules |> List.concat_map ~f:closure_node
-      | _ -> [ lib_interface ]
+        let closure =
+          Module_name.Map.values g.modules |> List.concat_map ~f:closure_node
+        in
+        lib_interface :: closure
+      | _ -> [ g.alias; lib_interface ]
 
     and closure_node = function
       | Module m -> [ m ]
@@ -489,7 +487,10 @@ module Group = struct
       Ok (if Module_name.equal name (Module.name li) then [ li ] else [])
     | _ ->
       (* TODO don't recompute this *)
-      let parents = parents t of_ |> List.map ~f:(fun g -> g.modules, Some g.name) in
+      let parents =
+        parents_modules [ t ] t.modules of_
+        |> List.map ~f:(fun g -> g.modules, Some g.name)
+      in
       Find_dep.find_dep_of_parents parents name
   ;;
 
@@ -791,7 +792,16 @@ let to_dyn t =
   | Stdlib s -> variant "Stdlib" [ Stdlib.to_dyn s ]
 ;;
 
-let lib ~obj_dir ~main_module_name ~wrapped ~stdlib ~lib_name ~implements ~modules =
+let lib
+      ~obj_dir
+      ~main_module_name
+      ~wrapped
+      ~stdlib
+      ~lib_name
+      ~implements
+      ~has_instances
+      ~modules
+  =
   let make_wrapped main_module_name =
     Wrapped
       (Wrapped.make ~obj_dir ~lib_name ~implements ~modules ~main_module_name ~wrapped)
@@ -809,7 +819,7 @@ let lib ~obj_dir ~main_module_name ~wrapped ~stdlib ~lib_name ~implements ~modul
          let mangle = Mangle.Unwrapped in
          Unwrapped (Unwrapped.of_trie modules ~mangle ~obj_dir)
        | (Yes_with_transition _ | Simple true), Some main_module_name, Some m ->
-         if Module.name m = main_module_name && not implements
+         if Module.name m = main_module_name && (not implements) && not has_instances
          then Singleton m
          else make_wrapped main_module_name
        | (Yes_with_transition _ | Simple true), Some main_module_name, None ->
@@ -836,15 +846,15 @@ let exe_unwrapped modules ~obj_dir =
   with_obj_map modules
 ;;
 
-let make_wrapped ~obj_dir ~modules kind =
+let make_wrapped ~obj_dir ~modules ~has_instances kind =
   let mangle : Mangle.t =
     match kind with
     | `Exe -> Exe
     | `Melange -> Melange
   in
   match Module_trie.as_singleton modules with
-  | Some m -> make_singleton m mangle
-  | None ->
+  | Some m when not has_instances -> make_singleton m mangle
+  | _ ->
     let modules = Wrapped (Wrapped.make_exe_or_melange ~obj_dir ~modules mangle) in
     with_obj_map modules
 ;;
@@ -920,6 +930,28 @@ let source_dirs =
     |> List.fold_left ~init:acc ~f:(fun acc f -> Path.Set.add acc (Path.parent_exn f)))
 ;;
 
+let compat_for_exn t m =
+  match t.modules with
+  | Singleton _ | Stdlib _ | Unwrapped _ -> assert false
+  | Wrapped { group; _ } ->
+    (match Module_name.Map.find group.modules (Module.name m) with
+     | None -> assert false
+     | Some (Module m) -> m
+     | Some (Group g) -> Group.lib_interface g)
+;;
+
+let entry_modules t =
+  List.filter
+    ~f:(fun m -> Module.visibility m = Public)
+    (match t.modules with
+     | Stdlib w -> Stdlib.lib_interface w |> Option.to_list
+     | Singleton m -> [ m ]
+     | Unwrapped m -> Unwrapped.entry_modules m
+     | Wrapped m ->
+       (* we assume this is never called for implementations *)
+       [ Wrapped.lib_interface m ])
+;;
+
 module With_vlib = struct
   type impl =
     { obj_map : Sourced_module.t Module_name.Unique.Map.t Lazy.t
@@ -1006,19 +1038,6 @@ module With_vlib = struct
     | Impl { impl = _; vlib; _ } -> lib_interface vlib
   ;;
 
-  let main_module_name =
-    let main_module_name t =
-      match t.modules with
-      | Singleton m -> Some (Module.name m)
-      | Unwrapped _ -> None
-      | Wrapped w -> Some w.group.name
-      | Stdlib w -> Some w.main_module_name
-    in
-    function
-    | Modules t -> main_module_name t
-    | Impl { vlib; impl = _; _ } -> main_module_name vlib
-  ;;
-
   let impl =
     let empty = lazy Module_name.Unique.Map.empty in
     fun impl ~vlib ->
@@ -1198,19 +1217,6 @@ module With_vlib = struct
     | Modules t -> { impl = fold t ~init ~f; vlib = [] }
   ;;
 
-  let compat_for_exn t m =
-    match t with
-    | Impl _ -> Code_error.raise "wrapped compat not supported for vlib" []
-    | Modules t ->
-      (match t.modules with
-       | Singleton _ | Stdlib _ | Unwrapped _ -> assert false
-       | Wrapped { group; _ } ->
-         (match Module_name.Map.find group.modules (Module.name m) with
-          | None -> assert false
-          | Some (Module m) -> m
-          | Some (Group g) -> Group.lib_interface g))
-  ;;
-
   let wrapped_compat t =
     match t with
     | Impl _ | Modules { modules = Stdlib _ | Singleton _ | Unwrapped _; _ } ->
@@ -1235,23 +1241,6 @@ module With_vlib = struct
     | Impl w -> Impl { w with impl = map w.impl }
   ;;
 
-  let entry_modules = function
-    | Impl i ->
-      Code_error.raise
-        "entry_modules: not defined for implementations"
-        [ "impl", dyn_of_impl i ]
-    | Modules t ->
-      List.filter
-        ~f:(fun m -> Module.visibility m = Public)
-        (match t.modules with
-         | Stdlib w -> Stdlib.lib_interface w |> Option.to_list
-         | Singleton m -> [ m ]
-         | Unwrapped m -> Unwrapped.entry_modules m
-         | Wrapped m ->
-           (* we assume this is never called for implementations *)
-           [ Wrapped.lib_interface m ])
-  ;;
-
   let wrapped = function
     | Modules t -> wrapped t
     | Impl { vlib = _; impl; _ } -> wrapped impl
