@@ -4,7 +4,7 @@
 open Import
 
 module Virtual : sig
-  type t = { virtual_modules : Ordered_set_lang.t }
+  type t = { virtual_modules : Ordered_set_lang.Unexpanded.t }
 end
 
 module Implementation : sig
@@ -18,13 +18,15 @@ type kind =
   | Virtual of Virtual.t
   | Implementation of Implementation.t
   | Exe_or_normal_lib
+  | Parameter
 
 val eval
-  :  modules:Module.Source.t Module_trie.t
+  :  expander:Expander.t
+  -> modules:Module.Source.t Module_trie.t
   -> stanza_loc:Loc.t
-  -> private_modules:Ordered_set_lang.t
+  -> private_modules:Ordered_set_lang.Unexpanded.t
   -> kind:kind
   -> src_dir:Path.Build.t
   -> version:Dune_lang.Syntax.Version.t
-  -> Stanza_common.Modules_settings.t
+  -> Modules_settings.t
   -> ((Loc.t * Module.Source.t) Module_trie.t * Module.t Module_trie.t) Memo.t
