@@ -30,7 +30,6 @@ open Import
 type 'path t
 
 val of_local : Path.Build.t t -> Path.t t
-
 val equal : 'a t -> 'a t -> bool
 
 (** The source_root directory *)
@@ -45,6 +44,8 @@ val native_dir : 'path t -> 'path
 (** The private compiled byte file directories, and all cmi *)
 val byte_dir : 'path t -> 'path
 
+val jsoo_dir : 'path t -> 'path
+
 (** The private compiled melange file directories, and all cmi *)
 val melange_dir : 'path t -> 'path
 
@@ -57,12 +58,11 @@ val public_cmi_ocaml_dir : 'path t -> 'path
 val public_cmi_melange_dir : 'path t -> 'path
 
 val odoc_dir : 'path t -> 'path
-
 val all_obj_dirs : 'path t -> mode:Lib_mode.t -> 'path list
 
 (** Create the object directory for a library *)
-val make_lib :
-     dir:Path.Build.t
+val make_lib
+  :  dir:Path.Build.t
   -> has_private_modules:bool
   -> private_lib:bool
   -> Lib_name.Local.t
@@ -73,21 +73,13 @@ val make_lib :
 val make_external_no_private : dir:Path.t -> Path.t t
 
 val encode : Path.t t -> Dune_lang.t list
-
 val decode : dir:Path.t -> Path.t t Dune_lang.Decoder.t
-
 val convert_to_external : Path.Build.t t -> dir:Path.t -> Path.t t
-
 val cm_dir : 'path t -> Lib_mode.Cm_kind.t -> Visibility.t -> 'path
-
 val to_dyn : _ t -> Dyn.t
-
 val make_exe : dir:Path.Build.t -> name:string -> Path.Build.t t
-
 val make_melange_emit : dir:Path.Build.t -> name:string -> Path.Build.t t
-
 val for_pp : dir:Path.Build.t -> Path.Build.t t
-
 val as_local_exn : Path.t t -> Path.Build.t t
 
 (** For local libraries with private modules, all public cmi's are symlinked to
@@ -103,30 +95,32 @@ module Module : sig
       .o files, ...) *)
 
   val cm_file : 'path t -> Module.t -> kind:Lib_mode.Cm_kind.t -> 'path option
+  val cm_public_file : 'path t -> Module.t -> kind:Lib_mode.Cm_kind.t -> 'path option
 
-  val cm_public_file :
-    'path t -> Module.t -> kind:Lib_mode.Cm_kind.t -> 'path option
-
-  val cmt_file :
-       'path t
+  val cmt_file
+    :  'path t
     -> Module.t
     -> ml_kind:Ml_kind.t
     -> cm_kind:Lib_mode.Cm_kind.t
     -> 'path option
 
-  val obj_file :
-    'path t -> Module.t -> kind:Lib_mode.Cm_kind.t -> ext:string -> 'path
+  val obj_file_of_name
+    :  'path t
+    -> Module_name.Unique.t
+    -> kind:Lib_mode.Cm_kind.t
+    -> ext:string
+    -> visibility:Visibility.t
+    -> 'path
+
+  val obj_file : 'path t -> Module.t -> kind:Lib_mode.Cm_kind.t -> ext:string -> 'path
 
   (** Same as [cm_file] but raises if [cm_kind] is [Cmo] or [Cmx] and the module
       has no implementation.*)
   val cm_file_exn : 'path t -> Module.t -> kind:Lib_mode.Cm_kind.t -> 'path
 
-  val o_file : 'path t -> Module.t -> ext_obj:string -> 'path option
-
-  val o_file_exn : 'path t -> Module.t -> ext_obj:string -> 'path
-
-  val cm_public_file_exn :
-    'path t -> Module.t -> kind:Lib_mode.Cm_kind.t -> 'path
+  val o_file : 'path t -> Module.t -> ext_obj:Filename.Extension.t -> 'path option
+  val o_file_exn : 'path t -> Module.t -> ext_obj:Filename.Extension.t -> 'path
+  val cm_public_file_exn : 'path t -> Module.t -> kind:Lib_mode.Cm_kind.t -> 'path
 
   (** Either the .cmti, or .cmt if the module has no interface *)
   val cmti_file : 'path t -> Module.t -> cm_kind:Lib_mode.Cm_kind.t -> 'path
@@ -134,17 +128,15 @@ module Module : sig
   val odoc : 'path t -> Module.t -> 'path
 
   module L : sig
-    val o_files : 'path t -> Module.t list -> ext_obj:string -> Path.t list
-
-    val cm_files :
-      'path t -> Module.t list -> kind:Lib_mode.Cm_kind.t -> Path.t list
+    val o_files : 'path t -> Module.t list -> ext_obj:Filename.Extension.t -> Path.t list
+    val cm_files : 'path t -> Module.t list -> kind:Lib_mode.Cm_kind.t -> Path.t list
   end
 
   module Dep : sig
     type t =
-      | Immediate of Module.File.t
+      | Immediate of Module.t * Ml_kind.t
       | Transitive of Module.t * Ml_kind.t
   end
 
-  val dep : Path.Build.t t -> Dep.t -> Path.Build.t
+  val dep : Path.Build.t t -> Dep.t -> Path.Build.t option
 end
