@@ -203,20 +203,26 @@ let odoc_ext = ".odoc"
 module Mld : sig
   type t
 
-  val create : Path.Build.t -> t
+  val create : path:Path.Build.t -> name:string -> t
   val odoc_file : doc_dir:Path.Build.t -> t -> Path.Build.t
   val odoc_input : t -> Path.Build.t
 end = struct
-  type t = Path.Build.t
+  (** The [(documentation (files ...))] stanza allows with the [as] keyword to
+      distinguish the input file and the path in the documentation. Here we do
+      not support layered hierarchy, but we do support changing the name (hence
+      the two fields) *)
+  type t =
+    { path : Path.Build.t
+    ; name : string (** The name of the mld compilation unit (without extension) *)
+    }
 
-  let create p = p
+  let create ~path ~name = { path; name }
 
-  let odoc_file ~doc_dir t =
-    let t = Filename.remove_extension (Path.Build.basename t) in
-    Path.Build.relative doc_dir (sprintf "page-%s%s" t odoc_ext)
+  let odoc_file ~doc_dir { name; _ } =
+    Path.Build.relative doc_dir (sprintf "page-%s%s" name odoc_ext)
   ;;
 
-  let odoc_input t = t
+  let odoc_input { path; _ } = path
 end
 
 module Flags = struct
@@ -248,11 +254,6 @@ let odoc_base_flags quiet build_dir =
   | Nonfatal -> S []
 ;;
 
-let odoc_dev_tool_lock_dir_exists () =
-  let path = Dune_pkg.Lock_dir.dev_tool_lock_dir_path Odoc in
-  Fs_memo.dir_exists (Path.Outside_build_dir.In_source_dir path)
-;;
-
 let odoc_dev_tool_exe_path_building_if_necessary () =
   let open Action_builder.O in
   let path = Path.build (Pkg_dev_tool.exe_path Odoc) in
@@ -261,9 +262,10 @@ let odoc_dev_tool_exe_path_building_if_necessary () =
 ;;
 
 let odoc_program sctx dir =
-  let open Action_builder.O in
-  let* odoc_dev_tool_lock_dir_exists =
-    Action_builder.of_memo (odoc_dev_tool_lock_dir_exists ())
+  let odoc_dev_tool_lock_dir_exists =
+    match Config.get Compile_time.lock_dev_tools with
+    | `Enabled -> true
+    | `Disabled -> false
   in
   match odoc_dev_tool_lock_dir_exists with
   | true -> odoc_dev_tool_exe_path_building_if_necessary ()
@@ -577,7 +579,7 @@ module Toplevel_index = struct
   (** This format is public API. *)
   let to_json items = `Assoc [ "packages", list_to_json items ~f:item_to_json ]
 
-  let json t = Dune_stats.Json.to_string (to_json t)
+  let json t = Json.to_string (to_json t)
 
   let content (output : Output_format.t) t =
     match output with
@@ -653,12 +655,11 @@ let create_odoc ctx ~target odoc_file =
 
 let check_mlds_no_dupes ~pkg ~mlds =
   match
-    List.rev_map mlds ~f:(fun mld ->
-      Filename.remove_extension (Path.Build.basename mld), mld)
+    List.rev_map mlds ~f:(fun ((_path, mld_name) as mld) -> mld_name, mld)
     |> Filename.Map.of_list
   with
   | Ok m -> m
-  | Error (_, p1, p2) ->
+  | Error (_, (p1, _name1), (p2, _name2)) ->
     User_error.raise
       [ Pp.textf
           "Package %s has two mld's with the same basename %s, %s"
@@ -668,20 +669,47 @@ let check_mlds_no_dupes ~pkg ~mlds =
       ]
 ;;
 
+let report_warnings warnings =
+  match warnings with
+  | [] -> ()
+  | _ :: _ ->
+    let l =
+      warnings
+      |> List.map ~f:(fun (mld : Doc_sources.mld) -> Path.Local.to_string mld.in_doc)
+      |> List.sort ~compare:String.compare
+      |> String.concat ~sep:", "
+    in
+    User_warning.emit
+      [ Pp.textf
+          "Dune does not yet support building documentation for assets, and mlds in a \
+           non-flat hierarchy. Ignoring %s."
+          l
+      ]
+;;
+
+let mlds sctx pkg =
+  let+ mlds = Packages.mlds sctx pkg in
+  List.partition_map mlds ~f:(fun (mld : Doc_sources.mld) ->
+    match Path.Local.explode mld.in_doc with
+    | [ name ] when String.equal (Filename.extension name) ".mld" ->
+      Left (mld.path, Filename.remove_extension name)
+    | _ -> Right mld)
+;;
+
 let odoc_artefacts sctx target =
   let ctx = Super_context.context sctx in
   let dir = Paths.odocs ctx target in
   match target with
   | Pkg pkg ->
     let+ mlds =
-      let+ mlds = Packages.mlds sctx pkg in
+      let+ mlds, _ = mlds sctx pkg in
       let mlds = check_mlds_no_dupes ~pkg ~mlds in
       Filename.Map.update mlds "index" ~f:(function
-        | None -> Some (Paths.gen_mld_dir ctx pkg ++ "index.mld")
+        | None -> Some (Paths.gen_mld_dir ctx pkg ++ "index.mld", "index")
         | Some _ as s -> s)
     in
-    Filename.Map.to_list_map mlds ~f:(fun _ mld ->
-      Mld.create mld |> Mld.odoc_file ~doc_dir:dir |> create_odoc ctx ~target)
+    Filename.Map.to_list_map mlds ~f:(fun _ (path, name) ->
+      Mld.create ~path ~name |> Mld.odoc_file ~doc_dir:dir |> create_odoc ctx ~target)
   | Lib lib ->
     let info = Lib.Local.info lib in
     let obj_dir = Lib_info.obj_dir info in
@@ -922,7 +950,8 @@ let package_mlds =
       ~input:(module Super_context.As_memo_key.And_package_name)
       (fun (sctx, pkg) ->
          Rules.collect (fun () ->
-           let* mlds = Packages.mlds sctx pkg in
+           let* mlds, warnings = mlds sctx pkg in
+           report_warnings warnings;
            let mlds = check_mlds_no_dupes ~pkg ~mlds in
            let ctx = Super_context.context sctx in
            if Filename.Map.mem mlds "index"
@@ -935,7 +964,7 @@ let package_mlds =
                  sctx
                  (Action_builder.write_file gen_mld (default_index ~pkg entry_modules))
              in
-             Filename.Map.set mlds "index" gen_mld)))
+             Filename.Map.set mlds "index" (gen_mld, "index"))))
   in
   fun sctx ~pkg -> Memo.exec memo (sctx, pkg)
 ;;
@@ -947,15 +976,15 @@ let setup_package_odoc_rules sctx ~pkg =
      back to a package name here. Need to try and change that one day. *)
   let* odocs =
     Filename.Map.values mlds
-    |> Memo.parallel_map ~f:(fun mld ->
+    |> Memo.parallel_map ~f:(fun (path, name) ->
       compile_mld
         sctx
-        (Mld.create mld)
+        (Mld.create ~path ~name)
         ~pkg
         ~doc_dir:(Paths.odocs ctx (Pkg pkg))
         ~includes:(Action_builder.return []))
   in
-  Dep.setup_deps ctx (Pkg pkg) (Path.set_of_build_paths_list odocs)
+  Path.Set.of_list_map ~f:Path.build odocs |> Dep.setup_deps ctx (Pkg pkg)
 ;;
 
 let gen_project_rules sctx project =
