@@ -398,10 +398,7 @@ module Valid = struct
   let valid_libs_and_packages =
     let run (ctx, all, projects) =
       let* libs_and_pkgs =
-        let* mask =
-          let+ mask = Dune_load.mask () in
-          Option.map ~f:Package.Name.Map.keys mask
-        in
+        let* mask = Dune_load.mask () in
         Scope.DB.with_all ctx ~f:(fun find ->
           Memo.List.fold_left projects ~init:([], []) ~f:(fun (libs_acc, pkg_acc) proj ->
             let* vendored = Source_tree.is_vendored (Dune_project.root proj) in
@@ -415,13 +412,13 @@ module Valid = struct
               let+ libs_acc =
                 let+ libs = Lib.DB.all lib_db in
                 let libs =
-                  match mask with
-                  | None -> libs
-                  | Some mask ->
+                  if Only_packages.mem_all mask
+                  then libs
+                  else
                     Lib.Set.filter libs ~f:(fun lib ->
                       let info = Lib.info lib in
                       match Lib_info.package info with
-                      | Some p -> List.mem ~equal:Package.Name.equal mask p
+                      | Some p -> Only_packages.mem mask p
                       | None -> false)
                 in
                 (proj, lib_db, libs) :: libs_acc
@@ -429,10 +426,9 @@ module Valid = struct
               let pkg_acc =
                 let pkgs =
                   let proj_pkgs = Dune_project.packages proj |> Package.Name.Map.keys in
-                  match mask with
-                  | None -> proj_pkgs
-                  | Some m ->
-                    List.filter ~f:(List.mem ~equal:Package.Name.equal m) proj_pkgs
+                  if Only_packages.mem_all mask
+                  then proj_pkgs
+                  else List.filter ~f:(Only_packages.mem mask) proj_pkgs
                 in
                 pkgs @ pkg_acc
               in
@@ -680,7 +676,7 @@ module Artifact : sig
   val module_name : t -> Module_name.t option
   val name : t -> string
   val make_module : Context.t -> all:bool -> Index.t -> Path.t -> visible:bool -> t
-  val external_mld : Context.t -> Index.t -> Path.t -> t
+  val external_mld : Context.t -> Index.t -> Path.t -> name:string -> t
   val index : Context.t -> all:bool -> Index.t -> t
 end = struct
   type artifact_ty =
@@ -755,30 +751,28 @@ end = struct
     v ~source ~odoc ~html_dir ~html_file:html ~ty:(Module visible)
   ;;
 
-  let int_make_mld ctx ~all index source ~is_index =
-    let basename = Path.basename source |> Filename.remove_extension in
+  let int_make_mld ctx ~all index source ~name ~is_index =
     let odoc =
       (if is_index then Index.obj_dir ctx ~all index else Index.odoc_dir ctx ~all index)
-      ++ ("page-" ^ basename ^ ".odoc")
+      ++ ("page-" ^ name ^ ".odoc")
     in
     let html_dir = Index.html_dir ctx ~all index in
-    let html =
-      html_dir ++ if is_index then "index.html" else sprintf "%s.html" basename
-    in
+    let html = html_dir ++ if is_index then "index.html" else sprintf "%s.html" name in
     v ~source ~odoc ~html_dir ~html_file:html ~ty:Mld
   ;;
 
-  let external_mld ctx index source =
-    int_make_mld ctx ~all:true index source ~is_index:false
+  let external_mld ctx index source ~name =
+    int_make_mld ctx ~all:true index source ~name ~is_index:false
   ;;
 
   let index ctx ~all index =
+    let filename = Index.mld_filename index in
     let source =
-      let filename = Index.mld_filename index in
       let dir = Index.obj_dir ctx ~all index in
       Path.build (dir ++ filename)
     in
-    int_make_mld ctx ~all index source ~is_index:true
+    let name = Filename.remove_extension filename in
+    int_make_mld ctx ~all index source ~name ~is_index:true
   ;;
 end
 
@@ -1245,7 +1239,7 @@ let ext_package_mlds (ctx : Context.t) (pkg : Package.Name.t) =
              | `File, dst ->
                let str = Install.Entry.Dst.to_string dst in
                if Filename.check_suffix str ".mld"
-               then Some (Path.relative doc_path str)
+               then Some (Path.relative doc_path str, str)
                else None
              | _ -> None))
       | _ -> None)
@@ -1255,7 +1249,10 @@ let ext_package_mlds (ctx : Context.t) (pkg : Package.Name.t) =
 let pkg_mlds sctx pkg =
   let* pkgs = Dune_load.packages () in
   if Package.Name.Map.mem pkgs pkg
-  then Packages.mlds sctx pkg >>| List.map ~f:Path.build
+  then
+    let+ res, warnings = Odoc.mlds sctx pkg in
+    let () = Odoc.report_warnings warnings in
+    List.map ~f:(fun (p, name) -> Path.build p, name) res
   else (
     let ctx = Super_context.context sctx in
     ext_package_mlds ctx pkg)
@@ -1263,11 +1260,11 @@ let pkg_mlds sctx pkg =
 
 let check_mlds_no_dupes ~pkg ~mlds =
   match
-    List.rev_map mlds ~f:(fun mld -> Filename.remove_extension (Path.basename mld), mld)
+    List.rev_map mlds ~f:(fun ((_path, mld_name) as mld) -> mld_name, mld)
     |> Filename.Map.of_list
   with
   | Ok m -> m
-  | Error (_, p1, p2) ->
+  | Error (_, (p1, _name1), (p2, _name2)) ->
     User_error.raise
       [ Pp.textf
           "Package %s has two mld's with the same basename %s, %s"
@@ -1284,12 +1281,14 @@ let pkg_artifacts sctx index pkg =
     check_mlds_no_dupes ~pkg ~mlds
   in
   let artifacts =
-    let mlds_noindex = String.Map.filteri ~f:(fun i _ -> i <> "index") mlds_map in
+    let mlds_noindex =
+      String.Map.filteri ~f:(fun i _ -> not (String.equal i "index")) mlds_map
+    in
     String.Map.values mlds_noindex
-    |> List.map ~f:(fun mld -> Artifact.external_mld ctx index mld)
+    |> List.map ~f:(fun (path, name) -> Artifact.external_mld ctx index path ~name)
   in
   let index_file = String.Map.find mlds_map "index" in
-  index_file, artifacts
+  Option.map ~f:fst index_file, artifacts
 ;;
 
 module Index_tree = struct
