@@ -39,13 +39,13 @@ end
 
 type t =
   { names : (Loc.t * string) Nonempty_list.t
-  ; link_flags : Link_flags.Spec.t
+  ; link_flags : Dune_lang.Link_flags.Spec.t
   ; link_deps : Dep_conf.t list
   ; modes : Loc.t Link_mode.Map.t
   ; optional : bool
   ; buildable : Buildable.t
   ; package : Package.t option
-  ; promote : Rule.Promote.t option
+  ; promote : Rule_mode.Promote.t option
   ; install_conf : Install_conf.t option
   ; embed_in_plugin_libraries : (Loc.t * Lib_name.t) list
   ; forbidden_libraries : (Loc.t * Lib_name.t) list
