@@ -76,6 +76,7 @@ type t =
   ; dune_version : Dune_lang.Syntax.Version.t
   ; virtual_modules : Ordered_set_lang.Unexpanded.t option
   ; implements : (Loc.t * Lib_name.t) option
+  ; parameters : (Loc.t * Lib_name.t) list
   ; default_implementation : (Loc.t * Lib_name.t) option
   ; private_modules : Ordered_set_lang.Unexpanded.t option
   ; stdlib : Ocaml_stdlib.t option
@@ -118,7 +119,37 @@ let decode =
          "modes"
          (Modes.decode ~stanza_loc ~dune_version project)
          ~default:(Mode_conf.Lib.Set.default stanza_loc)
-     and+ kind = field "kind" Lib_kind.decode ~default:Lib_kind.Normal
+     and+ virtual_modules, kind =
+       let* virtual_modules =
+         Ordered_set_lang.Unexpanded.field_o
+           ~check:(Dune_lang.Syntax.since Stanza.syntax (1, 7))
+           ~since_expanded:Modules_settings.since_expanded
+           "virtual_modules"
+       in
+       let+ dune_file_kind =
+         field "kind" Lib_kind.Dune_file.decode ~default:Lib_kind.Dune_file.Normal
+       in
+       let kind : Lib_kind.t =
+         match virtual_modules with
+         | None -> Dune_file dune_file_kind
+         | Some _ ->
+           (match dune_file_kind with
+            | Normal ->
+              (* Libraries are virtual just in case [virtual_modules] are specified
+                 and they do not have a *non-normal* kind specified. *)
+              Virtual
+            | (Ppx_deriver _ | Ppx_rewriter _) as incompatable_kind ->
+              User_error.raise
+                ~loc:stanza_loc
+                ~hints:[ Pp.text "Remove either the 'kind' or 'virtual_modules' fields" ]
+                [ Pp.text "Only virtual libraries can have 'virtual_modules'"
+                ; Pp.textf
+                    "but this library has 'virtual_modules' and is specified as kind \
+                     '%s'."
+                    (Lib_kind.Dune_file.cstr_name incompatable_kind)
+                ])
+       in
+       virtual_modules, kind
      and+ optional = field_b "optional"
      and+ no_dynlink = field_b "no_dynlink"
      and+ () =
@@ -136,15 +167,16 @@ let decode =
        let+ _ = field_b "no_keep_locs" ~check in
        ()
      and+ sub_systems = Sub_system_info.record_parser
-     and+ virtual_modules =
-       Ordered_set_lang.Unexpanded.field_o
-         ~check:(Dune_lang.Syntax.since Stanza.syntax (1, 7))
-         ~since_expanded:Stanza_common.Modules_settings.since_expanded
-         "virtual_modules"
      and+ implements =
        field_o
          "implements"
          (Dune_lang.Syntax.since Stanza.syntax (1, 7) >>> located Lib_name.decode)
+     and+ parameters =
+       field
+         "parameters"
+         (Dune_lang.Syntax.since Dune_lang.Oxcaml.syntax (0, 1)
+          >>> repeat (located Lib_name.decode))
+         ~default:[]
      and+ default_implementation =
        field_o
          "default_implementation"
@@ -152,7 +184,7 @@ let decode =
      and+ private_modules =
        Ordered_set_lang.Unexpanded.field_o
          ~check:(Dune_lang.Syntax.since Stanza.syntax (1, 2))
-         ~since_expanded:Stanza_common.Modules_settings.since_expanded
+         ~since_expanded:Modules_settings.since_expanded
          "private_modules"
      and+ stdlib =
        field_o
@@ -181,9 +213,7 @@ let decode =
          (Dune_lang.Syntax.since Stanza.syntax (2, 7)
           >>> fields (field "ppx" (located Lib_name.decode)))
      and+ package =
-       field_o
-         "package"
-         (Dune_lang.Syntax.since Stanza.syntax (2, 8) >>> located Stanza_common.Pkg.decode)
+       Stanza_pkg.field_opt ~check:(Dune_lang.Syntax.since Stanza.syntax (2, 8)) ()
      and+ melange_runtime_deps =
        field
          "melange.runtime_deps"
@@ -273,6 +303,7 @@ let decode =
      ; dune_version
      ; virtual_modules
      ; implements
+     ; parameters
      ; default_implementation
      ; private_modules
      ; stdlib
@@ -369,7 +400,7 @@ let best_name t =
   | Public p -> snd p.name
 ;;
 
-let is_virtual t = Option.is_some t.virtual_modules
+let is_virtual t = t.kind = Virtual
 let is_impl t = Option.is_some t.implements
 
 let obj_dir ~dir t =
@@ -384,12 +415,23 @@ let obj_dir ~dir t =
       ((* TODO instead of this fragile approximation, we should be looking at
           [Modules.t] and deciding. Unfortunately, [Obj_dir.t] is currently
           used in some places where [Modules.t] is not yet constructed. *)
-       t.private_modules <> None
-       || t.buildable.modules.root_module <> None)
+       t.private_modules
+       <> None
+          (* CR-someday rgrinberg: The following check used to be here:
+
+            {[
+              || t.buildable.modules.root_module <> None
+            ]}
+
+            but it doesn't work correctly. We need to always pass the
+            root_module with [ -H ] at least *)
+      )
     ~private_lib
     (snd t.name)
 ;;
 
+let local_main_module_name t = Some (Module_name.of_local_lib_name t.name)
+
 let main_module_name t : Lib_info.Main_module_name.t =
   match t.implements, t.wrapped with
   | Some x, From _ -> From x
@@ -421,7 +463,12 @@ let to_lib_info
   let archive ?(dir = dir) ext = archive conf ~dir ~ext in
   let modes = Mode_conf.Lib.Set.eval ~has_native conf.modes in
   let archive_for_mode ~f_ext ~mode =
-    if Mode.Dict.get modes.ocaml mode then Some (archive (f_ext mode)) else None
+    if Mode.Dict.get modes.ocaml mode
+    then (
+      match conf.kind with
+      | Parameter -> None
+      | Virtual | Dune_file _ -> Some (archive (f_ext mode)))
+    else None
   in
   let archives_for_mode ~f_ext =
     Mode.Dict.of_func (fun ~mode -> archive_for_mode ~f_ext ~mode |> Option.to_list)
@@ -485,6 +532,7 @@ let to_lib_info
       in
       archives_for_mode ~f_ext:Mode.compiled_lib_ext, plugins)
   in
+  let local_main_module_name = local_main_module_name conf in
   let main_module_name = main_module_name conf in
   let name = best_name conf in
   let lib_id =
@@ -524,6 +572,8 @@ let to_lib_info
     | Installed_private | Installed | Private _ -> None
   in
   let requires = conf.buildable.libraries in
+  let parameters = conf.parameters in
+  let allow_unused_libraries = conf.buildable.allow_unused_libraries in
   let loc = conf.buildable.loc in
   let kind = conf.kind in
   let src_dir = dir in
@@ -561,13 +611,16 @@ let to_lib_info
     ~version
     ~synopsis
     ~main_module_name
+    ~local_main_module_name
     ~sub_systems
     ~requires
+    ~parameters
     ~foreign_objects
     ~public_headers
     ~plugins
     ~archives
     ~ppx_runtime_deps
+    ~allow_unused_libraries
     ~foreign_archives
     ~native_archives
     ~foreign_dll_files
@@ -577,7 +630,6 @@ let to_lib_info
     ~enabled
     ~virtual_deps
     ~dune_version
-    ~virtual_
     ~entry_modules
     ~implements
     ~default_implementation
@@ -588,6 +640,7 @@ let to_lib_info
     ~exit_module
     ~instrumentation_backend
     ~melange_runtime_deps
+    ~root_module:(Option.map conf.buildable.modules.root_module ~f:snd)
 ;;
 
 include Stanza.Make (struct
