@@ -2,11 +2,8 @@ type t = int * int * int
 
 let make x = x
 let of_ocaml_config ocfg = Ocaml_config.version ocfg
-let supports_no_keep_locs version = version >= (4, 03, 0)
 let supports_opaque_for_mli version = version >= (4, 03, 0)
 let always_reads_alias_cmi version = version < (4, 03, 0)
-let supports_color_in_ocamlparam version = version >= (4, 03, 0)
-let supports_ocaml_color version = version >= (4, 05, 0)
 let supports_response_file version = version >= (4, 05, 0)
 let ocamlmklib_supports_response_file version = version >= (4, 08, 0)
 let stdlib_includes_bigarray version = version >= (4, 07, 0)
@@ -29,4 +26,13 @@ let supports_alerts version = version >= (4, 8, 0)
 let has_sandboxed_otherlibs version = version >= (5, 0, 0)
 let has_META_files version = version >= (5, 0, 0)
 let supports_bin_annot_occurrences version = version >= (5, 2, 0)
+let supports_hidden_includes version = version >= (5, 2, 0)
 let add_std_cxx_flag version = version >= (5, 0, 0)
+let supports_cmi_file version = version >= (5, 0, 0)
+
+let supports_oxcaml version =
+  let jst = "+jst" in
+  let ox = "+ox" in
+  Stdune.String.is_suffix ~suffix:jst version
+  || Stdune.String.is_suffix ~suffix:ox version
+;;
