@@ -1,6 +1,7 @@
 open Stdune
 module Checksum = Dune_pkg.Checksum
 module Lock_dir = Dune_pkg.Lock_dir
+module Dependency = Dune_pkg.Lock_dir.Dependency
 module Opam_repo = Dune_pkg.Opam_repo
 module Expanded_variable_bindings = Dune_pkg.Solver_stats.Expanded_variable_bindings
 module Package_variable_name = Dune_lang.Package_variable_name
@@ -64,15 +65,14 @@ module Update = struct
 end
 
 let lock_dir_encode_decode_round_trip_test ?commit ~lock_dir_path ~lock_dir () =
-  let lock_dir_path = Path.Source.of_string lock_dir_path in
+  let lock_dir_path = Path.of_string lock_dir_path in
   Lock_dir.Write_disk.(
-    prepare ~lock_dir_path ~files:Package_name.Map.empty lock_dir |> commit);
+    prepare ~portable_lock_dir:false ~lock_dir_path ~files:Package_name.Map.empty lock_dir
+    |> commit);
   let lock_dir_round_tripped =
     try Lock_dir.read_disk_exn lock_dir_path with
     | User_error.E _ as exn ->
-      let metadata_path =
-        Path.Source.relative lock_dir_path Lock_dir.metadata_filename |> Path.source
-      in
+      let metadata_path = Path.relative lock_dir_path Lock_dir.metadata_filename in
       let metadata_file_contents = Io.read_file metadata_path in
       print_endline
         "Failed to parse lockdir. Dumping raw metadata file to assist debugging.";
@@ -84,7 +84,9 @@ let lock_dir_encode_decode_round_trip_test ?commit ~lock_dir_path ~lock_dir () =
   in
   if Lock_dir.equal lock_dir_round_tripped' lock_dir'
   then print_endline "lockdir matches after roundtrip:"
-  else print_endline "lockdir doesn't match after roundtrip:";
+  else (
+    print_endline "lockdir doesn't match after roundtrip:";
+    print_endline (Lock_dir.to_dyn lock_dir |> Dyn.to_string));
   let dyn_lock_dir = Lock_dir.to_dyn lock_dir_round_tripped in
   let dyn_lock_dir =
     match commit with
@@ -111,7 +113,9 @@ let%expect_test "encode/decode round trip test for lockdir with no deps" =
          ~local_packages:[]
          ~ocaml:None
          ~repos:None
-         ~expanded_solver_variable_bindings:Expanded_variable_bindings.empty)
+         ~expanded_solver_variable_bindings:Expanded_variable_bindings.empty
+         ~solved_for_platform:None
+         ~portable_lock_dir:false)
     ();
   [%expect
     {|
@@ -123,13 +127,15 @@ let%expect_test "encode/decode round trip test for lockdir with no deps" =
     ; repos = { complete = true; used = None }
     ; expanded_solver_variable_bindings =
         { variable_values = []; unset_variables = [] }
-    } |}]
+    ; solved_for_platforms = ("<none>:1", [])
+    }
+    |}]
 ;;
 
 let empty_package name ~version =
-  { Lock_dir.Pkg.build_command = None
-  ; install_command = None
-  ; depends = []
+  { Lock_dir.Pkg.build_command = Lock_dir.Conditional_choice.empty
+  ; install_command = Lock_dir.Conditional_choice.empty
+  ; depends = Lock_dir.Conditional_choice.empty
   ; depexts = []
   ; info =
       { Lock_dir.Pkg_info.name
@@ -140,6 +146,7 @@ let empty_package name ~version =
       ; extra_sources = []
       }
   ; exported_env = []
+  ; enabled_on_platforms = []
   }
 ;;
 
@@ -160,6 +167,8 @@ let%expect_test "encode/decode round trip test for lockdir with simple deps" =
                [ Package_variable_name.os, Variable_value.string "linux" ]
            ; unset_variables = [ Package_variable_name.os_family ]
            }
+         ~solved_for_platform:None
+         ~portable_lock_dir:false
          (Package_name.Map.of_list_exn
             [ mk_pkg_basic ~name:"foo" ~version:(Package_version.of_string "0.1.0")
             ; mk_pkg_basic ~name:"bar" ~version:(Package_version.of_string "0.2.0")
@@ -173,35 +182,43 @@ let%expect_test "encode/decode round trip test for lockdir with simple deps" =
     ; packages =
         map
           { "bar" :
-              { build_command = None
-              ; install_command = None
-              ; depends = []
-              ; depexts = []
-              ; info =
-                  { name = "bar"
-                  ; version = "0.2.0"
-                  ; dev = false
-                  ; avoid = false
-                  ; source = None
-                  ; extra_sources = []
-                  }
-              ; exported_env = []
-              }
+              map
+                { "0.2.0" :
+                    { build_command = []
+                    ; install_command = []
+                    ; depends = []
+                    ; depexts = []
+                    ; info =
+                        { name = "bar"
+                        ; version = "0.2.0"
+                        ; dev = false
+                        ; avoid = false
+                        ; source = None
+                        ; extra_sources = []
+                        }
+                    ; exported_env = []
+                    ; enabled_on_platforms = []
+                    }
+                }
           ; "foo" :
-              { build_command = None
-              ; install_command = None
-              ; depends = []
-              ; depexts = []
-              ; info =
-                  { name = "foo"
-                  ; version = "0.1.0"
-                  ; dev = false
-                  ; avoid = false
-                  ; source = None
-                  ; extra_sources = []
-                  }
-              ; exported_env = []
-              }
+              map
+                { "0.1.0" :
+                    { build_command = []
+                    ; install_command = []
+                    ; depends = []
+                    ; depexts = []
+                    ; info =
+                        { name = "foo"
+                        ; version = "0.1.0"
+                        ; dev = false
+                        ; avoid = false
+                        ; source = None
+                        ; extra_sources = []
+                        }
+                    ; exported_env = []
+                    ; enabled_on_platforms = []
+                    }
+                }
           }
     ; ocaml = Some ("simple_lock_dir/lock.dune:3", "ocaml")
     ; repos = { complete = true; used = None }
@@ -209,6 +226,7 @@ let%expect_test "encode/decode round trip test for lockdir with simple deps" =
         { variable_values = [ ("os", "linux") ]
         ; unset_variables = [ "os-family" ]
         }
+    ; solved_for_platforms = ("<none>:1", [])
     }
     |}]
 ;;
@@ -216,6 +234,9 @@ let%expect_test "encode/decode round trip test for lockdir with simple deps" =
 let%expect_test "encode/decode round trip test for lockdir with complex deps" =
   let module Action = Dune_lang.Action in
   let module String_with_vars = Dune_lang.String_with_vars in
+  let make_conditional value =
+    Lock_dir.Conditional_choice.singleton Dune_pkg.Solver_env.empty value
+  in
   let lock_dir =
     let pkg_a =
       let name = Package_name.of_string "a" in
@@ -226,11 +247,11 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
       , let pkg = empty_package name ~version:(Package_version.of_string "0.1.0") in
         { pkg with
           build_command =
-            Some
-              (Action
+            make_conditional
+              (Lock_dir.Build_command.Action
                  Action.(Progn [ Echo [ String_with_vars.make_text Loc.none "hello" ] ]))
         ; install_command =
-            Some
+            make_conditional
               (Action.System
                  (* String_with_vars.t doesn't round trip so we have to set
                     [quoted] if the string would be quoted *)
@@ -260,8 +281,8 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
       ( name
       , let pkg = empty_package name ~version:(Package_version.of_string "dev") in
         { pkg with
-          install_command = None
-        ; depends = [ Loc.none, fst pkg_a ]
+          install_command = Lock_dir.Conditional_choice.empty
+        ; depends = make_conditional [ { Dependency.loc = Loc.none; name = fst pkg_a } ]
         ; info =
             { pkg.info with
               dev = true
@@ -283,7 +304,11 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
       ( name
       , let pkg = empty_package name ~version:(Package_version.of_string "0.2") in
         { pkg with
-          depends = [ Loc.none, fst pkg_a; Loc.none, fst pkg_b ]
+          depends =
+            make_conditional
+              [ { Dependency.loc = Loc.none; name = fst pkg_a }
+              ; { Dependency.loc = Loc.none; name = fst pkg_b }
+              ]
         ; info =
             { pkg.info with
               dev = false
@@ -304,6 +329,8 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
       ~ocaml:(Some (Loc.none, Package_name.of_string "ocaml"))
       ~repos:(Some [ opam_repo ])
       ~expanded_solver_variable_bindings:Expanded_variable_bindings.empty
+      ~solved_for_platform:None
+      ~portable_lock_dir:false
       (Package_name.Map.of_list_exn [ pkg_a; pkg_b; pkg_c ])
   in
   lock_dir_encode_decode_round_trip_test ~lock_dir_path:"complex_lock_dir" ~lock_dir ();
@@ -315,63 +342,98 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
     ; packages =
         map
           { "a" :
-              { build_command = Some (Action [ "progn"; [ "echo"; "hello" ] ])
-              ; install_command = Some [ "system"; "echo 'world'" ]
-              ; depends = []
-              ; depexts = []
-              ; info =
-                  { name = "a"
-                  ; version = "0.1.0"
-                  ; dev = false
-                  ; avoid = false
-                  ; source = Some { url = "file:///tmp/a"; checksum = None }
-                  ; extra_sources =
-                      [ ("one", { url = "file:///tmp/a"; checksum = None })
-                      ; ("two", { url = "file://randomurl"; checksum = None })
-                      ]
-                  }
-              ; exported_env = [ { op = "="; var = "foo"; value = "bar" } ]
-              }
-          ; "b" :
-              { build_command = None
-              ; install_command = None
-              ; depends = [ ("complex_lock_dir/b.pkg:3", "a") ]
-              ; depexts = []
-              ; info =
-                  { name = "b"
-                  ; version = "dev"
-                  ; dev = true
-                  ; avoid = false
-                  ; source =
-                      Some
-                        { url = "https://github.com/foo/b"
-                        ; checksum =
-                            Some
-                              "sha256=adfc38f14c0188a2ad80d61451d011d27ab8839b717492d7ad42f7cb911c54c3"
+              map
+                { "0.1.0" :
+                    { build_command =
+                        [ { condition = [ map {} ]
+                          ; value = Action [ "progn"; [ "echo"; "hello" ] ]
+                          }
+                        ]
+                    ; install_command =
+                        [ { condition = [ map {} ]
+                          ; value = [ "system"; "echo 'world'" ]
+                          }
+                        ]
+                    ; depends = []
+                    ; depexts = []
+                    ; info =
+                        { name = "a"
+                        ; version = "0.1.0"
+                        ; dev = false
+                        ; avoid = false
+                        ; source =
+                            Some { url = "file:///tmp/a"; checksum = None }
+                        ; extra_sources =
+                            [ ("one", { url = "file:///tmp/a"; checksum = None })
+                            ; ("two",
+                               { url = "file://randomurl"; checksum = None })
+                            ]
                         }
-                  ; extra_sources = []
-                  }
-              ; exported_env = []
-              }
+                    ; exported_env = [ { op = "="; var = "foo"; value = "bar" } ]
+                    ; enabled_on_platforms = []
+                    }
+                }
+          ; "b" :
+              map
+                { "dev" :
+                    { build_command = []
+                    ; install_command = []
+                    ; depends =
+                        [ { condition = [ map {} ]
+                          ; value =
+                              [ { loc = "complex_lock_dir/b.pkg:3"; name = "a" }
+                              ]
+                          }
+                        ]
+                    ; depexts = []
+                    ; info =
+                        { name = "b"
+                        ; version = "dev"
+                        ; dev = true
+                        ; avoid = false
+                        ; source =
+                            Some
+                              { url = "https://github.com/foo/b"
+                              ; checksum =
+                                  Some
+                                    "sha256=adfc38f14c0188a2ad80d61451d011d27ab8839b717492d7ad42f7cb911c54c3"
+                              }
+                        ; extra_sources = []
+                        }
+                    ; exported_env = []
+                    ; enabled_on_platforms = []
+                    }
+                }
           ; "c" :
-              { build_command = None
-              ; install_command = None
-              ; depends =
-                  [ ("complex_lock_dir/c.pkg:3", "a")
-                  ; ("complex_lock_dir/c.pkg:3", "b")
-                  ]
-              ; depexts = []
-              ; info =
-                  { name = "c"
-                  ; version = "0.2"
-                  ; dev = false
-                  ; avoid = false
-                  ; source =
-                      Some { url = "https://github.com/foo/c"; checksum = None }
-                  ; extra_sources = []
-                  }
-              ; exported_env = []
-              }
+              map
+                { "0.2" :
+                    { build_command = []
+                    ; install_command = []
+                    ; depends =
+                        [ { condition = [ map {} ]
+                          ; value =
+                              [ { loc = "complex_lock_dir/c.pkg:3"; name = "a" }
+                              ; { loc = "complex_lock_dir/c.pkg:3"; name = "b" }
+                              ]
+                          }
+                        ]
+                    ; depexts = []
+                    ; info =
+                        { name = "c"
+                        ; version = "0.2"
+                        ; dev = false
+                        ; avoid = false
+                        ; source =
+                            Some
+                              { url = "https://github.com/foo/c"
+                              ; checksum = None
+                              }
+                        ; extra_sources = []
+                        }
+                    ; exported_env = []
+                    ; enabled_on_platforms = []
+                    }
+                }
           }
     ; ocaml = Some ("complex_lock_dir/lock.dune:3", "ocaml")
     ; repos =
@@ -380,6 +442,7 @@ let%expect_test "encode/decode round trip test for lockdir with complex deps" =
         }
     ; expanded_solver_variable_bindings =
         { variable_values = []; unset_variables = [] }
+    ; solved_for_platforms = ("<none>:1", [])
     }
     |}]
 ;;
@@ -414,6 +477,8 @@ let%expect_test "encode/decode round trip test with locked repo revision" =
         ~ocaml:(Some (Loc.none, Package_name.of_string "ocaml"))
         ~repos:(Some [ opam_repo ])
         ~expanded_solver_variable_bindings:Expanded_variable_bindings.empty
+        ~solved_for_platform:None
+        ~portable_lock_dir:false
         (Package_name.Map.of_list_exn [ pkg_a; pkg_b; pkg_c ])
     in
     lock_dir_encode_decode_round_trip_test
@@ -429,50 +494,62 @@ let%expect_test "encode/decode round trip test with locked repo revision" =
     ; packages =
         map
           { "a" :
-              { build_command = None
-              ; install_command = None
-              ; depends = []
-              ; depexts = []
-              ; info =
-                  { name = "a"
-                  ; version = "0.1.0"
-                  ; dev = false
-                  ; avoid = false
-                  ; source = None
-                  ; extra_sources = []
-                  }
-              ; exported_env = []
-              }
+              map
+                { "0.1.0" :
+                    { build_command = []
+                    ; install_command = []
+                    ; depends = []
+                    ; depexts = []
+                    ; info =
+                        { name = "a"
+                        ; version = "0.1.0"
+                        ; dev = false
+                        ; avoid = false
+                        ; source = None
+                        ; extra_sources = []
+                        }
+                    ; exported_env = []
+                    ; enabled_on_platforms = []
+                    }
+                }
           ; "b" :
-              { build_command = None
-              ; install_command = None
-              ; depends = []
-              ; depexts = []
-              ; info =
-                  { name = "b"
-                  ; version = "dev"
-                  ; dev = false
-                  ; avoid = false
-                  ; source = None
-                  ; extra_sources = []
-                  }
-              ; exported_env = []
-              }
+              map
+                { "dev" :
+                    { build_command = []
+                    ; install_command = []
+                    ; depends = []
+                    ; depexts = []
+                    ; info =
+                        { name = "b"
+                        ; version = "dev"
+                        ; dev = false
+                        ; avoid = false
+                        ; source = None
+                        ; extra_sources = []
+                        }
+                    ; exported_env = []
+                    ; enabled_on_platforms = []
+                    }
+                }
           ; "c" :
-              { build_command = None
-              ; install_command = None
-              ; depends = []
-              ; depexts = []
-              ; info =
-                  { name = "c"
-                  ; version = "0.2"
-                  ; dev = false
-                  ; avoid = false
-                  ; source = None
-                  ; extra_sources = []
-                  }
-              ; exported_env = []
-              }
+              map
+                { "0.2" :
+                    { build_command = []
+                    ; install_command = []
+                    ; depends = []
+                    ; depexts = []
+                    ; info =
+                        { name = "c"
+                        ; version = "0.2"
+                        ; dev = false
+                        ; avoid = false
+                        ; source = None
+                        ; extra_sources = []
+                        }
+                    ; exported_env = []
+                    ; enabled_on_platforms = []
+                    }
+                }
           }
     ; ocaml = Some ("complex_lock_dir/lock.dune:3", "ocaml")
     ; repos =
@@ -485,6 +562,7 @@ let%expect_test "encode/decode round trip test with locked repo revision" =
         }
     ; expanded_solver_variable_bindings =
         { variable_values = []; unset_variables = [] }
+    ; solved_for_platforms = ("<none>:1", [])
     }
     |}]
 ;;
