@@ -160,7 +160,7 @@ let%expect_test "related error" =
         ]
       ; [ "message"
         ; [ "Verbatim"
-          ; "The implementation foo.ml does not match the interface foo.ml: \n\
+          ; "The implementation foo.ml does not match the interface foo.mli: \n\
              Values do not match: val x : bool is not included in val x : int\n\
              The type bool is not compatible with the type int\n\
              "
@@ -520,7 +520,7 @@ let%expect_test "create and fix error" =
           ]
         ; [ "message"
           ; [ "Verbatim"
-            ; "This expression has type int but an expression was expected of type\n\
+            ; "The constant 123 has type int but an expression was expected of type\n\
               \  string\n\
                "
             ]
@@ -563,7 +563,7 @@ let%expect_test "create and fix error" =
           ]
         ; [ "message"
           ; [ "Verbatim"
-            ; "This expression has type int but an expression was expected of type\n\
+            ; "The constant 123 has type int but an expression was expected of type\n\
               \  string\n\
                "
             ]
@@ -814,6 +814,86 @@ let g = A.f
           ; [ "severity"; "error" ]
           ; [ "targets"; [] ]
           ]
-        ] |}]);
-  [%expect {||}]
+        ] |}])
+;;
+
+let%expect_test "cyclic dependency error simple" =
+  setup_diagnostics (fun client ->
+    files
+      [ "dune", "(executable (name foo))"; "foo.ml", "open Bar"; "bar.ml", "open Foo" ];
+    let* poll = poll_exn client Dune_rpc.Public.Sub.diagnostic in
+    let* () = print_diagnostics poll in
+    [%expect
+      {|
+        <no diagnostics> |}];
+    let* () = dune_build client "./foo.exe" in
+    [%expect
+      {|
+        Building ./foo.exe
+        Build ./foo.exe failed |}];
+    let+ () = print_diagnostics poll in
+    [%expect
+      {|
+      [ "Add"
+      ; [ [ "id"; "0" ]
+        ; [ "message"
+          ; [ "Verbatim"
+            ; "Error: dependency cycle between modules in _build/default:\n\
+              \   Foo\n\
+               -> Bar\n\
+               -> Foo\n\
+               "
+            ]
+          ]
+        ; [ "promotion"; [] ]
+        ; [ "related"; [] ]
+        ; [ "severity"; "error" ]
+        ; [ "targets"; [] ]
+        ]
+      ]
+      |}])
+;;
+
+let%expect_test "cyclic dependency error" =
+  setup_diagnostics (fun client ->
+    files
+      [ "dune", "(executable (name foo))"
+      ; "foo.ml", "open Bar open Baz"
+      ; "bar.ml", "open Baz"
+      ; "baz.ml", "open Bar"
+      ];
+    let* poll = poll_exn client Dune_rpc.Public.Sub.diagnostic in
+    let* () = print_diagnostics poll in
+    [%expect
+      {|
+        <no diagnostics> |}];
+    let* () = dune_build client "./foo.exe" in
+    [%expect
+      {|
+        Building ./foo.exe
+        Build ./foo.exe failed |}];
+    let+ () = print_diagnostics poll in
+    let backtrace_regex = Re.str "\\ " |> Re.compile in
+    [%expect.output]
+    |> String.split_lines
+    |> List.filter ~f:(fun s -> not (Re.execp backtrace_regex s))
+    |> String.concat ~sep:"\n"
+    |> print_endline;
+    [%expect
+      {|
+      [ "Add"
+      ; [ [ "id"; "0" ]
+        ; [ "message"
+          ; [ "Verbatim"
+            ; "Cycle_error.E\n\
+               "
+            ]
+          ]
+        ; [ "promotion"; [] ]
+        ; [ "related"; [] ]
+        ; [ "severity"; "error" ]
+        ; [ "targets"; [] ]
+        ]
+      ]
+      |}])
 ;;
